@@ -16,15 +16,17 @@
 **
 ****************************************************************************/
 #include "calculator.h"
+#include "deprecation.h"
+#include "itsme.h"
 
-Calculator::Calculator(double wavel, QWidget *parent):
-    QWidget(parent)
-{
+Calculator::Calculator(QString cellLine, QWidget *parent):
+    QWidget(parent){
     setWindowFlags(Qt::Window);
     setWindowTitle(tr("Calculator"));
 // Set up wavelength box
 //    double lambda;
 //    QList<QPair<QString,double> > wavelist;
+    setLocale(QLocale::C);
     wavelist = new QList<XraySource>();
     wllist = new QStringList();
     xraytype = new QStringList();
@@ -44,8 +46,7 @@ Calculator::Calculator(double wavel, QWidget *parent):
     wavelist->push_back(XraySource(XraySource::In));
     wavelist->push_back(XraySource(XraySource::CUSTOM)); // Should be last in this array !
 
-    for(QList<XraySource>::iterator it = wavelist->begin();it!=wavelist->end();it++ )
-    {
+    for(QList<XraySource>::iterator it = wavelist->begin();it!=wavelist->end();it++ ){
         if(it->name() == "INPUT")
             wllist->push_back("From file");
         else
@@ -75,6 +76,98 @@ Calculator::Calculator(double wavel, QWidget *parent):
 // Put tab widget and bottons into the dialog
     mainvlayout->addWidget(tabwidget);
     mainvlayout->addWidget(CloseButton,0,Qt::AlignRight);
+
+// hkl resolution tab widget
+    QWidget *hklResolutionWidget = new QWidget(this);
+    QGridLayout *hklgrid = new QGridLayout(hklResolutionWidget);
+    lambdaSpin = new QDoubleSpinBox(this);
+    lambdaSpin->setRange(0.01,10.0);
+    lambdaSpin->setDecimals(5);
+    lambdaSpin->setSingleStep(0.0001);
+    aSpin = new QDoubleSpinBox(this);
+    aSpin->setRange(1.00,500.0);
+    aSpin->setDecimals(5);
+    aSpin->setSingleStep(0.01);
+    bSpin = new QDoubleSpinBox(this);
+    bSpin->setRange(1.0,500.0);
+    bSpin->setDecimals(5);
+    bSpin->setSingleStep(0.01);
+    cSpin = new QDoubleSpinBox(this);
+    cSpin->setRange(1.0,500.0);
+    cSpin->setDecimals(5);
+    cSpin->setSingleStep(0.01);
+    alSpin = new QDoubleSpinBox(this);
+    alSpin->setRange(1.0,180.0);
+    alSpin->setDecimals(4);
+    alSpin->setSingleStep(0.1);
+    beSpin = new QDoubleSpinBox(this);
+    beSpin->setRange(1.0,180.0);
+    beSpin->setDecimals(4);
+    beSpin->setSingleStep(0.1);
+    gaSpin = new QDoubleSpinBox(this);
+    gaSpin->setRange(1.0,180.0);
+    gaSpin->setDecimals(4);
+    gaSpin->setSingleStep(0.1);
+    hSpin = new QSpinBox(this);
+    hSpin->setRange(-2000,2000);
+    hSpin->setValue(1);
+
+    kSpin = new QSpinBox(this);
+    kSpin->setRange(-2000,2000);
+    kSpin->setValue(1);
+
+    lSpin = new QSpinBox(this);
+    lSpin->setRange(-2000,2000);
+    lSpin->setValue(1);
+
+    int j(0);
+    LCell = new QLineEdit((cellLine.isEmpty())?"CELL 0.5 10.0 10.0 10.0 90.0 90.0 90.0":cellLine.simplified(),this);
+    hklgrid->addWidget(LCell,j,0,1,3);
+    QPushButton *getCurentCell = new QPushButton("get current cell",this);
+    connect(getCurentCell,SIGNAL(pressed()),this,SIGNAL(currentCellrequest()));
+    hklgrid->addWidget(getCurentCell,j,3);
+    HKLedit = new QLineEdit("HKL 1 1 1",this);
+    hklgrid->addWidget(new QLabel("wavelength",this),++j,0,Qt::AlignRight);
+    hklgrid->addWidget(lambdaSpin,j,1);
+    hklgrid->addWidget(HKLedit,j,2,1,2);
+    hklgrid->addWidget(new QLabel("a",this),++j,0,Qt::AlignRight);
+    hklgrid->addWidget(aSpin,j,1);
+    hklgrid->addWidget(new QLabel("h",this),j,2,Qt::AlignRight);
+    hklgrid->addWidget(hSpin,j,3);
+    hklgrid->addWidget(new QLabel("b",this),++j,0,Qt::AlignRight);
+    hklgrid->addWidget(bSpin,j,1);
+    hklgrid->addWidget(new QLabel("k",this),j,2,Qt::AlignRight);
+    hklgrid->addWidget(kSpin,j,3);
+    hklgrid->addWidget(new QLabel("c",this),++j,0,Qt::AlignRight);
+    hklgrid->addWidget(cSpin,j,1);
+    hklgrid->addWidget(new QLabel("l",this),j,2,Qt::AlignRight);
+    hklgrid->addWidget(lSpin,j,3);
+    hklgrid->addWidget(new QLabel("alpha",this),++j,0,Qt::AlignRight);
+    hklgrid->addWidget(alSpin,j,1);
+    hklgrid->addWidget(new QLabel("beta",this),++j,0,Qt::AlignRight);
+    hklgrid->addWidget(beSpin,j,1);
+    hklgrid->addWidget(new QLabel("gamma",this),++j,0,Qt::AlignRight);
+    hklgrid->addWidget(gaSpin,j,1);
+    resolutionL=new QLabel("",this);
+    hklgrid->addWidget(resolutionL,++j,0,1,4,Qt::AlignRight);
+    wlcellChanged();
+    hkl2stl2();
+
+    connect(LCell,SIGNAL(textEdited(QString)),this,SLOT(wlcellChanged()));
+    connect(lambdaSpin,SIGNAL(valueChanged(double)),this,SLOT(changeWlCell()));
+    connect(aSpin,SIGNAL(valueChanged(double)),this,SLOT(changeWlCell()));
+    connect(bSpin,SIGNAL(valueChanged(double)),this,SLOT(changeWlCell()));
+    connect(cSpin,SIGNAL(valueChanged(double)),this,SLOT(changeWlCell()));
+    connect(alSpin,SIGNAL(valueChanged(double)),this,SLOT(changeWlCell()));
+    connect(beSpin,SIGNAL(valueChanged(double)),this,SLOT(changeWlCell()));
+    connect(gaSpin,SIGNAL(valueChanged(double)),this,SLOT(changeWlCell()));
+    connect(hSpin,SIGNAL(valueChanged(int)),this,SLOT(hkl2stl2()));
+    connect(kSpin,SIGNAL(valueChanged(int)),this,SLOT(hkl2stl2()));
+    connect(lSpin,SIGNAL(valueChanged(int)),this,SLOT(hkl2stl2()));
+    connect(HKLedit,SIGNAL(textEdited(QString)),this,SLOT(hKLChanged()));
+    connect(hSpin,SIGNAL(valueChanged(int)),this,SLOT(changeHKLEdit()));
+    connect(kSpin,SIGNAL(valueChanged(int)),this,SLOT(changeHKLEdit()));
+    connect(lSpin,SIGNAL(valueChanged(int)),this,SLOT(changeHKLEdit()));
 // Add resolution page to the tab widget
     QWidget *reswidget = new QWidget();
     resgrid = new QGridLayout(); // a grid for resolution
@@ -115,10 +208,17 @@ Calculator::Calculator(double wavel, QWidget *parent):
 // put resolution and 1-3 distance widgets to the tab
     tabwidget->insertTab(0,reswidget,tr("Resolution"));
     tabwidget->insertTab(1,dangwidget,tr("1-3 Distances"));
+    tabwidget->insertTab(2,hklResolutionWidget,QString("Resolution of reflection HKL"));
 
     danglayout->insertLayout(0,danggrid);
     dangpicturelabel = new QLabel(this);
-    dangpicturelabel->setPixmap(QPixmap(":/xle-icons/dang.png"));
+    QString theIconPath;
+    if (!QString(PROGRAM_NAME).compare("shelXle")){
+        theIconPath=QString(":/xle-icons/");
+    }else{
+        theIconPath=QString(":/bicons/");
+    }
+    dangpicturelabel->setPixmap(QPixmap(theIconPath+"dang.png"));
     dangpicturelabel->show();
     danglayout->insertWidget(1,dangpicturelabel);
 // Initiate all widgets for 1-3 distances tab
@@ -155,13 +255,14 @@ Calculator::Calculator(double wavel, QWidget *parent):
 Default sinA/L = 0.6
 */
     _sintl = 0.6;
-    _shelxlambda = lambda = wavel;
+    QString s = LCell->text();
+    QRegularExpression sep=QRegularExpression("\\s+");
+    _shelxlambda = lambda = (s.isEmpty())?0.71073:s.section(sep,1,1).toDouble();
     sinthoverlmline->setText(QString::number(_sintl));
     sinthoverlmline_changed();
     waveline->setText(QString::number(lambda,10,5));
 
-    if(_shelxlambda < 0.001)
-    {
+    if(_shelxlambda < 0.001)    {
         xraytypebox->setEnabled(false);
         waveline->setEnabled(false);
     }
@@ -187,13 +288,15 @@ Default sinA/L = 0.6
 
     connect(CloseButton, SIGNAL(clicked()), this, SLOT(close()));
 //    connect(CancelButton, SIGNAL(clicked()), this, SLOT(close()));
-
-    setFixedSize(sizeHint());
+    QSize _size=sizeHint();
+    _size.setWidth(static_cast<int>(_size.width()*1.2));
+    setFixedSize(_size);
     show();
+
+
 }
 
-Calculator::~Calculator()
-{
+Calculator::~Calculator(){
     delete CloseButton;
     delete wllist;
     delete wavelist;
@@ -207,11 +310,11 @@ Calculator::~Calculator()
     delete d2line;
     delete angleline;
     delete dline;
+    printf("~Calculator()\n");
 }
 /*! \brief Recalculate all values in the line using dresline as a reference
 */
-void Calculator::dresline_changed()
-{
+void Calculator::dresline_changed(){
 /* 2dsinA = L
    sinA/L = 1/(2d)
    (sinA/L)^2 = 1/(2d)^2
@@ -228,8 +331,7 @@ void Calculator::dresline_changed()
     twothetaline->setText(QString::number(_twotheta,10,2));
 }
 
-void Calculator::sinthoverlmline_changed()
-{
+void Calculator::sinthoverlmline_changed(){
 /*    sinA/L = 1/(2d) = s
            d =  1/(2s)
   (sinA/L)^2 = s^2
@@ -246,8 +348,134 @@ void Calculator::sinthoverlmline_changed()
     twothetaline->setText(QString::number(_twotheta,10,2));
 }
 
-void Calculator::thetaline_changed()
-{
+void Calculator::changeWlCell(){
+  QString s=QString("CELL %1 %2 %3 %4 %5 %6 %7")
+          .arg(lambdaSpin->value(),0,'f',5)
+          .arg(aSpin->value()     ,0,'g',5)
+          .arg(bSpin->value()     ,0,'g',5)
+          .arg(cSpin->value()     ,0,'g',5)
+          .arg(alSpin->value()    ,0,'g',4)
+          .arg(beSpin->value()    ,0,'g',4)
+          .arg(gaSpin->value()    ,0,'g',4);
+  if (!LCell->hasFocus()) {
+      LCell->setText(s);
+
+      mole.cell.wave = lambdaSpin->value();
+      mole.cell.a =    aSpin->value()     ;
+      mole.cell.b =    bSpin->value()     ;
+      mole.cell.c =    cSpin->value()     ;
+      mole.cell.al =   alSpin->value()    ;
+      mole.cell.be =   beSpin->value()    ;
+      mole.cell.ga =   gaSpin->value()    ;
+      mole.cellSetup();
+  QSize _size=size();
+#if (QT_VERSION>=0x050000)
+    int w=75+
+    fontMetrics().horizontalAdvance(s);
+
+#else
+    int w=75+
+    fontMetrics().width(s);
+#endif
+  _size.setWidth(static_cast<int>(qMax(w,width())));
+  setFixedSize(_size);
+  printf("changeWlCell w%d\n",qMax(w,width()));
+  }
+  hkl2stl2();
+}
+
+void Calculator::setHKLEdit(QString text){
+    LCell->setFocus();
+    LCell->setText(text.simplified());
+    wlcellChanged();
+}
+
+void Calculator::wlcellChanged(){
+  QString s=LCell->text();
+  if (s.isEmpty())return;
+  QRegularExpression sep=QRegularExpression("\\s+");
+  QStringList tok = s.split(sep,skipEmptyParts);
+  if (tok.size()<8)return;
+  mole.cell.wave = tok.at(1).toDouble();
+  mole.cell.a =    tok.at(2).toDouble();
+  mole.cell.b =    tok.at(3).toDouble();
+  mole.cell.c =    tok.at(4).toDouble();
+  mole.cell.al =   tok.at(5).toDouble();
+  mole.cell.be =   tok.at(6).toDouble();
+  mole.cell.ga =   tok.at(7).toDouble();
+  mole.cellSetup();
+  lambdaSpin->setValue(mole.cell.wave);
+  aSpin->setValue(mole.cell.a);
+  bSpin->setValue(mole.cell.b);
+  cSpin->setValue(mole.cell.c);
+  alSpin->setValue(mole.cell.al);
+  beSpin->setValue(mole.cell.be);
+  gaSpin->setValue(mole.cell.ga);
+  //printf("\n!!%f %f %f %f %f %f %f %d\n",mole.cell.wave,mole.cell.a,mole.cell.b,mole.cell.c,mole.cell.al,mole.cell.be,mole.cell.ga,tok.size());
+  QSize _size=size();
+#if (QT_VERSION>=0x050000)
+    int w=75+
+    fontMetrics().horizontalAdvance(s);
+
+#else
+    int w=75+
+    fontMetrics().width(s);
+#endif
+  _size.setWidth(static_cast<int>(qMax(w,width())));
+  setFixedSize(_size);
+  //hkl2stl2();
+}
+
+void Calculator::hKLChanged(){
+    QString s=HKLedit->text();
+    printf("hKLChanged() %d\n",s.isEmpty());
+    if (s.isEmpty())return;
+    QRegularExpression sep=QRegularExpression("\\s+");
+    QStringList tok = s.split(sep, skipEmptyParts);
+    if ((tok.size()==3)&&(!tok.at(0).contains("HKL",Qt::CaseInsensitive))){
+        hSpin->setValue(tok.at(0).toInt());
+        kSpin->setValue(tok.at(1).toInt());
+        lSpin->setValue(tok.at(2).toInt());
+        QString s=QString("HKL %1 %2 %3")
+                .arg(hSpin->value()     )
+                .arg(kSpin->value()     )
+                .arg(lSpin->value()     );
+        HKLedit->setText(s);
+        hkl2stl2();
+        return;
+    }
+    if (tok.size()<4)return;
+    hSpin->setValue(tok.at(1).toInt());
+    kSpin->setValue(tok.at(2).toInt());
+    lSpin->setValue(tok.at(3).toInt());
+    hkl2stl2();
+}
+
+void Calculator::changeHKLEdit(){
+    QString s=QString("HKL %1 %2 %3")
+            .arg(hSpin->value()     )
+            .arg(kSpin->value()     )
+            .arg(lSpin->value()     );
+    if (!HKLedit->hasFocus()) {
+        HKLedit->setText(s);
+        hkl2stl2();
+    }
+}
+
+void Calculator::hkl2stl2(){
+    int h = hSpin->value();
+    int k = kSpin->value();
+    int l = lSpin->value();
+    V3 H=V3(h,k,l);
+    double stl=sqrt(H*mole.cell.Gi*H)/2.0;
+   // printf("%f d=%f 2theta=%f\n",stl,0.5/stl,2.0*RAD2DEG(asin(stl*mole.cell.wave)));
+    resolutionL->setText(QString("<h2>d = %1 &Aring;; sin(&theta;/&lambda;) = %2 &Aring;<sup>-1</sup>; 2&theta; = %3 &#176; </h2>")
+                         .arg(0.5/stl,0,'g',5)
+                         .arg(stl,0,'g',5)
+                         .arg(2.0*RAD2DEG(asin(stl*mole.cell.wave)),0,'f',2));
+}
+
+void Calculator::thetaline_changed(){
 /*           theta = s
              d = L/(2sin(s))
              sin(s)/L
@@ -264,8 +492,7 @@ void Calculator::thetaline_changed()
     twothetaline->setText(QString::number(_twotheta,10,2));
 }
 
-void Calculator::twothetaline_changed()
-{
+void Calculator::twothetaline_changed(){
     /*           2theta = s
                  d = L/(2 sin(s/2))
                  sin(s)/L
@@ -282,8 +509,7 @@ void Calculator::twothetaline_changed()
     thetaline->setText(QString::number(_theta,10,2));
 }
 
-void Calculator::wl_changed(int a)
-{
+void Calculator::wl_changed(int a){
 /*
     2dsinA = L
     sinA/L = 1/(2d) = s
@@ -331,8 +557,7 @@ void Calculator::wl_changed(int a)
     twothetaline->setText(QString::number(_twotheta,10,2));
 }
 
-void Calculator::wl_type_changed(int a)
-{
+void Calculator::wl_type_changed(int a){
     lambda = wavelist->at(wavebox->currentIndex()).wavelength(static_cast<XraySource::XrayType>(a));
     waveline->setText(QString::number(lambda,10,5));
 
@@ -342,8 +567,7 @@ void Calculator::wl_type_changed(int a)
     twothetaline->setText(QString::number(_twotheta,10,2));
 }
 
-void Calculator::wl_custom_changed()
-{
+void Calculator::wl_custom_changed(){
    lambda = waveline->text().toDouble();
    _theta = RAD2DEG(asin(lambda/(2*_dres)));
    _twotheta = 2*_theta;
@@ -351,8 +575,7 @@ void Calculator::wl_custom_changed()
    twothetaline->setText(QString::number(_twotheta,10,2));
 }
 
-void Calculator::d1line_changed()
-{
+void Calculator::d1line_changed(){
     _d1 = d1line->text().toDouble();
 //    _d2 = d2line->text().toDouble();
 //    _angle = angleline->text().toDouble();
@@ -360,8 +583,7 @@ void Calculator::d1line_changed()
     dline->setText(QString::number(_d,10,3));
 }
 
-void Calculator::d2line_changed()
-{
+void Calculator::d2line_changed(){
 //    _d1 = d1line->text().toDouble();
     _d2 = d2line->text().toDouble();
 //    _angle = angleline->text().toDouble();
@@ -370,8 +592,7 @@ void Calculator::d2line_changed()
 
 }
 
-void Calculator::angleline_changed()
-{
+void Calculator::angleline_changed(){
 //    _d1 = d1line->text().toDouble();
 //    _d2 = d2line->text().toDouble();
     _angle = angleline->text().toDouble();
@@ -379,8 +600,7 @@ void Calculator::angleline_changed()
     dline->setText(QString::number(_d,10,3));
 }
 
-void Calculator::dline_changed()
-{
+void Calculator::dline_changed(){
 //    _d1 = d1line->text().toDouble();
 //    _d2 = d2line->text().toDouble();
     _d = dline->text().toDouble();
@@ -389,15 +609,13 @@ void Calculator::dline_changed()
 }
 
 
-XraySource::XraySource(XraySource::Sources source)
-{
+XraySource::XraySource(XraySource::Sources source){
 /*
 All wavelengths were takedn from
 Internationl Union of Crystallography Vol. C (2004)
 Table 4.2.2.1
 */
-    switch(source)
-    {
+    switch(source)    {
     case XraySource::INPUT:
         _name = "INPUT";
 //        _wavelength = 0;
@@ -517,8 +735,7 @@ Table 4.2.2.1
     }
 }
 
-double XraySource::wavelength(XraySource::XrayType type) const
-{
+double XraySource::wavelength(XraySource::XrayType type) const{
     double res(0);
 
     switch (type) {
