@@ -19,8 +19,15 @@
 #ifndef CHGL_H
 #define CHGL_H
 
-#include <QGLWidget>
-
+#include <QOpenGLWidget>
+#include <QToolButton>
+#include <QCheckBox>
+#include <QComboBox>
+#include <QDoubleSpinBox>
+#include <QSpinBox>
+#include <QLabel>
+#include <QApplication>
+#include <QGridLayout>
 #include "molecule.h"
 //#include "fourxle.h"
 #ifndef MAXSELECT
@@ -35,7 +42,7 @@
 #define HIDE_REASON_BELO           32
 /*! \brief ChGL is the QGLWidget of ShelXle
  */
-class ChGL : public QGLWidget{
+class ChGL : public QOpenGLWidget{
 Q_OBJECT
 public:
     explicit ChGL(QWidget * parent = 0);
@@ -47,6 +54,7 @@ public:
     QTimer *idl_end;//! idle rotation timer
     bool on;    
     bool habzutun;
+    QString theIconPath;
     int ww,//!<Window with
 	wh,//!<Window height
 	minus;//!<for streo mode swiches right and left eye view    
@@ -109,7 +117,7 @@ public:
         *nocntrPlot,
         *rainbowPlot,
         *hideBeLo,//!< hide/show  BEDE and LONE objects
-        *useTextureLabels,//!< alternative to Qt renderText render Labels on BillBoard textures
+        //*useTextureLabels,//!< alternative to Qt renderText render Labels on BillBoard textures
         *shortLabels,//!< Labels without residue number or symmery flag are considered short
         *atomLegend;//!< a key legend for atoms used
     QToolButton *enviButt;//!< Button to clear the envi -ronment.
@@ -137,17 +145,27 @@ public:
     V3 altemitte;//!<rotation center position
     V3 altcenter;
     bool altpivot;
-    QComboBox *pivot1cb;
-    QComboBox *pivot2cb;
-    QDoubleSpinBox *axx;
-    QDoubleSpinBox *axy;
-    QDoubleSpinBox *axz;
-    QCheckBox *manualAx;
-    QLabel *whyNotSplit;
+    QComboBox *pivot1cb=nullptr;
+    QComboBox *pivot2cb=nullptr;
+    QDoubleSpinBox *axx=nullptr;
+    QDoubleSpinBox *axy=nullptr;
+    QDoubleSpinBox *axz=nullptr;
+    QDoubleSpinBox *orgx=nullptr;
+    QDoubleSpinBox *orgy=nullptr;
+    QDoubleSpinBox *orgz=nullptr;
+    QDoubleSpinBox *pickRadSpin=nullptr;
+    QCheckBox *manualAx=nullptr;
+    QCheckBox *freeRot=nullptr;
+    QLabel *whyNotSplit=nullptr;
 //    void initLists();
     bool canSplitRotate();
-    QSpinBox *angle1;
-    QSpinBox *angle2;
+    double pickRad(){
+        return sqrt(pickradius);
+    }
+    QSpinBox *angle1=nullptr;
+    QSpinBox *angle2=nullptr;
+    void qglColor(const QColor c);
+    void qglClearColor(const QColor c);
     void farbverlauf(double wrt, double min, double max);
     QColor farbverlaufQC(double wrt, double min, double max);
     void zoom(double speed);
@@ -206,6 +224,7 @@ signals:
     void insertANIS(QList<MyAtom> selected); //!< Insert ANIS instruction for selected atoms
     void pivot1Changed(int i);
     void pivot2Changed(int i);
+    void initialized();
 public slots:
     void sdm();
     void rotateIdle();
@@ -220,12 +239,15 @@ public slots:
 //    void clearLists();
     void connectSelection();
     void crosseye();
-    void decFontSize();
     void disConnectSelection(int index);
     void disSelection();
     void envi();
     void expand();
-    void fertig(){habzutun=false;updateGL();printf("map veritces ready!\n");}
+    void fertig(){
+        habzutun=false;
+        update();
+        //printf("map veritces ready!\n");
+    }
     void hardwareStereo();
     void hideHydrogens();
     void hideNonSelected();
@@ -236,9 +258,8 @@ public slots:
     void hideThisFragment();
     void highliteQPeak(double co);
     void homeXY();
-    void incFontSize();
     void invertSelection();
-    void invertHidden();//!< Toggle visibility of atoms etc
+    void invertHidden();//!< Toggle visibility of atoms etc    
     void Listen();
     void singleBondDistance();
     void loadOrientation();
@@ -264,13 +285,18 @@ public slots:
     void setMatrix();
     void setMolecule(Molecule *m);
     void setReNaMo(bool b);
+    void setPickRad(double r){
+        pickradius = r*r;
+        showPickRad=true;
+        if (!pause) update();
+    }
     void setRotationCenter();
     void setRotationCenter(int rz);
     void setRotationCenter(V3 center);
     void setTube(bool b){
       //!sets tube mode to b
       tubes=b;
-      updateGL();
+      if (!pause) update();
     }
     void setUnit(bool b);
     void setViewAngle(double ang);
@@ -282,7 +308,7 @@ public slots:
     void unsetTube(bool b){
       //!sets tube mode to not b
       tubes=!b;
-      updateGL();
+      if (!pause) update();
     }
     void updateLabelTextures();
     void zalman();
@@ -305,29 +331,43 @@ protected:
    void initializeGL();
    void resizeGL(int width, int height);
    void paintGL();
-#if (QT_VERSION >= 0x040600) && (defined (Q_WS_MAC) || defined(Q_OS_MAC))
-   bool event(QEvent *event);
-#endif
+//#if (QT_VERSION >= 0x040600) && (defined (Q_WS_MAC) || defined(Q_OS_MAC))
+//   bool event(QEvent *event);
+//#endif
    void mousePressEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    void contextMenuEvent(QContextMenuEvent *event);
    void wheelEvent(QWheelEvent *event);  
 private:
   QWidget *chparent; 
+  bool was_short_labels = false;
+  QStringList labelTexts;
   inline void __RotateCS( double c, double s, double& X, double& Y ) ;
   void glTranslateL( const double dx, const double dy, const double dz );
   void glRotateL( const double dang, const double x, const double y, const double z );
-  void findPivot();  
+  void glRotateMat( Matrix &mat, const double dang, const double x, const double y, const double z );
+  void findPivot();
+  void renderText(double x, double y, double z, const QString &str);
+  void renderText(double x, double y, const QString &str, int an);
+  void project(double x, double y, double z, GLdouble &tx, GLdouble &ty);
+  bool prepareShaderProgram(const QString& vertexShaderPath,
+                            const QString& geometryShaderPath,
+                            const QString& fragmentShaderPath);
+
+  QOpenGLShaderProgram m_shader;
+  int angleH, colorH;
+  GLfloat ang;
   QList<int> seat;//selected atoms indices for splitrotation
   int pivot,pivot2;
   int ppp,pp,p;//there is no need this should longer GLuint 26.02.2020
-#if (QT_VERSION >= 0x040600) && (defined (Q_WS_MAC) || defined(Q_OS_MAC))
-        bool gestureEvent(QGestureEvent *event);
-        void pinchTriggered(QPinchGesture*);
-        void swipeTriggered(QSwipeGesture*);
-#endif
+//#if (QT_VERSION >= 0x040600) && (defined (Q_WS_MAC) || defined(Q_OS_MAC))
+        //bool gestureEvent(QGestureEvent *event);
+        //void pinchTriggered(QPinchGesture*);
+        //void swipeTriggered(QSwipeGesture*);
+//#endif
         bool mouseOverInteraction,atomsClickable,tubes,bggradient,depthcueing;
         bool drawAx,drawUc,drawAt,drawADP,drawBo,drawLa,drawHb,drawFO,drawDF, warLabel;
+        bool showPickRad=false;
         int imFokus,afok;
         void draw();
         Connection apair;
@@ -336,9 +376,9 @@ private:
         double qcutoff,pickradius;
         int frid;
         double labScal;
-        QList<QImage> labelTextures;
-        QList<QImage> extraTextures;
-        QList<QImage> enviTextures;
+        QList<QOpenGLTexture*> labelTextures;
+        QList<QOpenGLTexture*> extraTextures;
+        QList<QOpenGLTexture*> enviTextures;
         int withsymm;
         QPoint lastPos;
         QStringList resis();
