@@ -44,7 +44,17 @@
 
 #include <QPlainTextEdit>
 #include <QObject>
+#include <QGroupBox>
+#include <QLineEdit>
+#include <QToolButton>
+#include <QToolButton>
+#include <QCheckBox>
+#include <QGridLayout>
+#include <QComboBox>
+#include <QGraphicsScene>
+
 #include "highlighter.h"
+#include "multitextcursor.h"
 
 #if defined Q_WS_WIN || defined Q_OS_WIN
 #define FONTFAM "Lucida Console"
@@ -66,6 +76,11 @@ QT_END_NAMESPACE
 class LineNumberArea;
 class AfixHighlightArea;
 
+static inline bool isPrintableText(const QString &text) {
+    return !text.isEmpty() && (text.at(0).isPrint() || text.at(0) == QLatin1Char('\t'));
+}
+
+
 /*! \brief CodeEditor is the editor of shelXle featuring syntax higlighter, code completer, LineNumberArea, AfixHighlightArea and Part Higlighter.
  *
  *
@@ -77,11 +92,16 @@ class CodeEditor : public QPlainTextEdit {
     CodeEditor(QWidget *parent = 0);
     ~CodeEditor();
     QList<int> sfac;//!<List of Scattering factors.
+    QStringList sfacsmbl;//!<List of Scattering factors pte symbols (needed as D has same atomic number as H)
+    QStringList easf;//!Electron Atom Scattering Factors (SFAC)
+    QStringList xasf;//!X-Ray Atom Scattering Factors (SFAC)
     int sortierWeise;
     int midCursorpos;
     bool ListFile;
     bool uCanSeeMe;
     bool dark;
+    double smin;
+    double smax;
     QString unitAlt, unitNeu;
     //QSyntaxHighlighter *highlighter;//!< Highlighter is the syntax higlighter. 
     Highlighter *highlighter;//!< Highlighter is the syntax higlighter. 
@@ -101,16 +121,103 @@ class CodeEditor : public QPlainTextEdit {
     QList<bool> comment;//!< List of all lines true if they are comments
     QList<double> fv;//!< List of 'Free Variables' in the SHELX res or ins file. 
     int fvMinLine;//!< fvMinLine specifies the line number of the FVAR istruction. 
-    QGroupBox  *suchbox;
-    QLineEdit *searchLE, *replace;
+    QGroupBox   *suchbox;
+    QLineEdit   *searchLE, *replace;
     QToolButton *hidesearch;
     QToolButton *prev,*next,*replaceButton,*replacePButton,*replaceAButton; 
-    QCheckBox *wholeLine;
+    QCheckBox   *wholeLine;
     QGridLayout *slt;
-    QComboBox *resiFinder;
+    QComboBox    *resiFinder;
+    QString dispFromWave(int an);
     QString sortLabelList(CEnvironment &asymm, MyAtom &tatze);
     void searchchanged(bool current,bool back);
   //void searchchanged2(bool current,bool back);
+    MultiTextCursor multiTextCursor() const;
+    void setMultiTextCursor(const MultiTextCursor &cursor);
+    QRect cursorUpdateRect(const MultiTextCursor &cursor);
+    void doSetTextCursor1(const QTextCursor &cursor);
+    void doSetTextCursor(const QTextCursor &cursor, bool keepMultiSelection);
+    //void setExtraSelections(Utils::Id kind, const QList<QTextEdit::ExtraSelection> &selections);
+    //    QHash<Utils::Id, QList<QTextEdit::ExtraSelection>> m_extraSelections;
+
+    void startCursorFlashTimer();
+    void resetCursorFlashTimer();
+    QBasicTimer m_cursorFlashTimer;
+    bool m_cursorVisible;
+    //bool m_moveLineUndoHack = false;
+    void updateCursorSelections();
+    void moveCursor(QTextCursor::MoveOperation operation,  QTextCursor::MoveMode mode = QTextCursor::MoveAnchor);
+    bool cursorMoveKeyEvent(QKeyEvent *e);
+    void handleHomeKey(bool anchor, bool block);
+    void handleBackspaceKey();
+    struct BlockSelection{
+        int blockNumber;
+        int column;
+        int anchorBlockNumber;
+        int anchorColumn;
+    };
+    QList<BlockSelection> m_blockSelections;
+    QList<QTextCursor> generateCursorsForBlockSelection(const BlockSelection &blockSelection);
+    void initBlockSelection();
+    void clearBlockSelection();
+    void handleMoveBlockSelection(QTextCursor::MoveOperation op);
+    struct CursorData
+    {
+        QTextLayout *layout;
+        QPointF offset;
+        int pos;
+        QPen pen;
+    };
+
+    struct PaintEventData{
+        PaintEventData(CodeEditor *editor, QPaintEvent *event, QPointF offset)
+            : offset(offset)
+            , viewportRect(editor->viewport()->rect())
+            , eventRect(event->rect())
+            , doc(editor->document())
+            , documentLayout(qobject_cast<QPlainTextDocumentLayout *>(doc->documentLayout()))
+            , documentWidth(int(doc->size().width()))
+            , textCursor(editor->textCursor())
+            , textCursorBlock(textCursor.block())
+            , isEditable(!editor->isReadOnly())
+        { }
+        QPointF offset;
+        const QRect viewportRect;
+        const QRect eventRect;
+        qreal rightMargin;
+        const QTextDocument *doc;
+        QPlainTextDocumentLayout *documentLayout;
+        const int documentWidth;
+        const QTextCursor textCursor;
+        const QTextBlock textCursorBlock;
+        const bool isEditable;
+        const QTextCharFormat searchScopeFormat;
+        const QTextCharFormat searchResultFormat;
+        const QTextCharFormat visualWhitespaceFormat;
+        const QTextCharFormat ifdefedOutFormat;
+        QAbstractTextDocumentLayout::PaintContext context;
+        QTextBlock  visibleCollapsedBlock;
+        QPointF visibleCollapsedBlockOffset;
+        QTextBlock block;
+        QList<CursorData> cursors;
+    };//PaintEventData
+    struct PaintEventBlockData{
+        QRectF boundingRect;
+        QVector<QTextLayout::FormatRange> selections;
+        QTextLayout *layout;
+        int position;
+        int length;
+    };
+    void setupBlockLayout(const PaintEventData &data, QPainter &painter, PaintEventBlockData &blockData) const;
+    void setupSelections(const PaintEventData &data, PaintEventBlockData &blockData) const;
+    void addCursorsPosition(PaintEventData &data, QPainter &painter, const PaintEventBlockData &blockData) const;
+    void paintCursor(const PaintEventData &data, QPainter &painter) const;
+    static bool blockContainsCursor(const PaintEventBlockData &blockData, const QTextCursor &cursor);
+    static CursorData generateCursorData(const int cursorPos,
+                                         const PaintEventData &data,
+                                         const PaintEventBlockData &blockData,
+                                         QPainter &painter);
+
 signals:
     void findInStructure(const QString &);//!< findInStructure is emitted by a context menu to select and center an atom in the OpenGL representation of the structure.
     void openIncludeFile(const QString &);
@@ -121,19 +228,37 @@ signals:
 protected:
     void resizeEvent(QResizeEvent *event);
     bool viewportEvent(QEvent *event);
-    void keyPressEvent(QKeyEvent *e);
+    void keyPressEvent(QKeyEvent *e);    
+    void mouseMoveEvent(QMouseEvent *);
+    void mousePressEvent(QMouseEvent *);    
+    void timerEvent(QTimerEvent *);
     void focusInEvent(QFocusEvent *e);
     void contextMenuEvent(QContextMenuEvent *event);
+    void focusOutEvent(QFocusEvent *e);
+    void paintEvent(QPaintEvent *);
 public slots:
+    void testCpd();
+    void mpaste();
+    void mcut();
+    void plotSFAC();
+    void slotCursorPositionChanged();
+    void updateCursorPosition();
+    void handleCBstates(int  state);
     void vis(bool b){ uCanSeeMe=b; }
+    void fitWidth();
+    void restoreMinimumWidth();
     void updateLineNumberAreaWidth(int newBlockCount);//!< updateLineNumberAreaWidth sets the new width of LineNumberArea for the given amount of lines newBlockCount. @param newBlockCount number of lines in the editor.
     void insertAnis();//!< inserts ANIS instruction into the file.
     void insertActa();//!< inserts ACTA instruction into the file.
+    void insertCONF();//!< inserts CONF instruction into the file.
     void insertList4();//!< inserts a LIST 4 instruction into the file.
     void insertList6();//!< inserts a LIST 6 instruction into the file.
     void insertBind();//!< inserts a BIND instruction into the file.
+    void insertEXTI();//!< inserts a EXTI instruction into the file.
     void insertFree();//!< inserts a FREE instruction into the file.
-    void findResi(QString);//!<Searches a Residue via a string like "RESI 22 ALA" or "RESI ALA 22" in the editor and selects all atoms with the same residue number (eg: 22).
+    void insertElectronSFACs();
+    void insertXRaySFACs();
+    void findResi(int);//!<Searches a Residue via a string like "RESI 22 ALA" or "RESI ALA 22" in the editor and selects all atoms with the same residue number (eg: 22).
     void findText();//!<Find functionality of search and replace in the editor.
     void hideSearch();//!< Hides the search and replace widget for the editor.
     void replaceAll();//!<replace all functionality of search and replace in the editor.
@@ -188,6 +313,10 @@ private:
     QCompleter *c;
     QWidget *afixHighlightArea;
     ChGL *chgl;
+    bool cbhandled=true;
+    MultiTextCursor m_cursors;
+    bool startMouseMoveCursor1;
+    double cromerMann(double sintl, double a1, double b1,  double a2,  double b2, double a3, double b3, double a4, double b4, double c);
 };
 
 /////////////////////////////////////////////////
@@ -244,4 +373,34 @@ class LineNumberArea : public QWidget{
     CodeEditor *codeEditor;
 };
 
+class CheckBox : public QCheckBox{
+public:
+    CheckBox(QGraphicsScene * scene, const QString & text, QWidget * parent = 0);
+protected:
+    void hoverEnter(QHoverEvent *event);
+    void hoverLeave(QHoverEvent *event);
+    void hoverMove(QHoverEvent *event);
+    bool event(QEvent *event){
+        switch (event->type()){
+        case QEvent::HoverEnter:
+            hoverEnter(static_cast<QHoverEvent*>(event));
+            return true;
+            break;
+        case QEvent::HoverLeave:
+            hoverLeave(static_cast<QHoverEvent*>(event));
+            return true;
+            break;
+        case QEvent::HoverMove:
+            hoverMove(static_cast<QHoverEvent*>(event));
+            return true;
+            break;
+        default:
+            break;
+        }
+        return QWidget::event(event);
+    }
+private:
+    QGraphicsScene *scene;
+};
+
 #endif
