@@ -2,6 +2,9 @@
 #define DRAGDROPATOMSDLG_H
 
 #include <QtGui>
+#include <QDialog>
+#include <QPushButton>
+#include <QScrollArea>
 #include "molecule.h"
 #include "chgl.h"
 /*! \brief DragAtom is an atom label that can be draged in the 'Inherit Labels' feature. 
