@@ -10,8 +10,9 @@
 
 #include "dsrglwindow.h"
 #include "molecule.h"
-#include "chgl.h"
+//#include "chgl.h"
 #include "dsrgui.h"
+#include "glureplace.h"
 
 
 DSRGlWindow::DSRGlWindow(QWidget *parent, Molecule *m,
@@ -45,6 +46,7 @@ DSRGlWindow::DSRGlWindow(QWidget *parent, Molecule *m,
   mole.pmin=10000;
   mole.pmax=-10000;
   mole.LOD=3;
+  labScal=0.4;
   // initial zoom level:
   // The Zom will later be adapted to fragment size.
   L=50.0;
@@ -60,6 +62,8 @@ DSRGlWindow::DSRGlWindow(QWidget *parent, Molecule *m,
 }
 
 DSRGlWindow::~DSRGlWindow() {
+    mole.g_Program->release();
+    mole.g_Program->deleteLater();
 }
 
 static inline void transform_point(GLdouble out[4], const GLdouble m[16], const GLdouble in[4]) {
@@ -78,7 +82,7 @@ static inline void transform_point(GLdouble out[4], const GLdouble m[16], const
 static inline bool  posTo2D(V3 obj,
 		const GLdouble model[16], const GLdouble proj[16],
 		const GLint viewport[4],
-		GLdouble * winx, GLdouble * winy) {
+        GLdouble * winx, GLdouble * winy,int retinafktr=1) {
   GLdouble in[4], out[4];
 
   in[0] = obj.x;
@@ -95,19 +99,48 @@ static inline bool  posTo2D(V3 obj,
   in[2] /= in[3];
 
   *winx = viewport[0] + (1 + in[0]) * viewport[2] / 2;
-  *winy = viewport[1] + (1 - in[1]) * viewport[3] / 2;
+  *winy = viewport[1] + (1 - in[1]) * viewport[3] / 2;  
+  *winx /= retinafktr;
+  *winy /= retinafktr;
   return true;
 }
 
+void DSRGlWindow::project(double x, double y, double z, GLdouble &tx, GLdouble &ty){
+  makeCurrent();
+  GLdouble model[16];
+  GLdouble proj[16];
+  GLint viewport[4];
+  glGetIntegerv(GL_VIEWPORT, viewport);
+  glGetDoublev( GL_PROJECTION_MATRIX , (double*)proj );
+  glGetDoublev( GL_MODELVIEW_MATRIX, (double*)model );
+  V3 pos(x,y,z);
+  posTo2D(pos, model, proj, viewport, &tx, &ty, retinafktr);
+}
+
+void DSRGlWindow::renderText(double x, double y, double z, const QString &text){
+    // Identify x and y locations to render text within widget
+  GLdouble textPosX = 0.0, textPosY = 0.0;
+  project(x, y, z, textPosX, textPosY);
+  //printf("%f %f %f -> %f %f\n",x,y,z,textPosX,textPosY);
+  // Render text
+  QPainter painter(this);
+  QFontMetrics mtr(myFont);
+  QRect rc=mtr.boundingRect(text+"L");
+  QImage image(rc.width(),rc.height(), QImage::Format_ARGB32_Premultiplied);
+  image.fill(0);
+  QPainter p;
+  p.begin(&image);
+  p.setPen(dsrLabelColor);
+  p.setBackground(Qt::NoBrush);
+  p.setFont(myFont);
+  p.drawText(image.rect(), Qt::AlignCenter, text);
+  p.end();
+  painter.drawImage(textPosX, textPosY-rc.height(), image);
+  painter.end();
+}
+
 
 void DSRGlWindow::draw(){
-  /*const GLfloat  OBJ_SPE[]   = { 0.8, 0.8, 0.8, 1.0 };
-    const GLfloat  OBJ_SHIN    = 127.0;               
-    glMaterialfv( GL_FRONT_AND_BACK, GL_SPECULAR,             OBJ_SPE  );
-    glEnable     ( GL_COLOR_MATERIAL ) ;
-    glColorMaterial ( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE ) ;    
-    glMaterialf(  GL_FRONT_AND_BACK, GL_SHININESS,           OBJ_SHIN );
-  */
   mole.dratom = 0;
   glPushMatrix();
   glScaled( L, L, L );
@@ -115,22 +148,41 @@ void DSRGlWindow::draw(){
   GLdouble proj[16];
   GLint viewport[4];
   glGetIntegerv(GL_VIEWPORT, viewport);
-#if defined Q_OS_MAC && (QT_VERSION >= 0x050000)
-  viewport[2]/=2;
-  viewport[3]/=2;
-#endif
-  //      printf("%d %d %d %d\n",viewport[0],viewport[1],viewport[2],viewport[3]);
+  viewport[2]/=retinafktr;
+  viewport[3]/=retinafktr;
   glGetDoublev( GL_PROJECTION_MATRIX, (double*)proj );
   glGetDoublev( GL_MODELVIEW_MATRIX, (double*)model );
+  /*printf("The Mmatrix is:\n%9.6f %9.6f %9.6f %9.6f\n%9.6f %9.6f %9.6f %9.6f\n%9.6f %9.6f %9.6f %9.6f\n%9.6f %9.6f %9.6f %9.6f\n",
+      model[0],model[1],model[2],model[3],
+      model[4],model[5],model[6],model[7],
+      model[8],model[9],model[10],model[11],
+      model[12],model[13],model[14],model[15]);
+  GLdouble det=
+    model[0]*model[5]*model[10] - model[8]*model[5]*model[2]+
+    model[1]*model[6]*model[8]  - model[9]*model[6]*model[0]+
+    model[2]*model[4]*model[9]  - model[10]*model[4]*model[1];
+  printf("die Determinante der Drehmatrix ist: %g\n",det);
+  printf("Die Pmatrix ist:\n%9.6f %9.6f %9.6f %9.6f\n%9.6f %9.6f %9.6f %9.6f\n%9.6f %9.6f %9.6f %9.6f\n%9.6f %9.6f %9.6f %9.6f\n",
+      proj[0],proj[1],proj[2],proj[3],
+      proj[4],proj[5],proj[6],proj[7],
+      proj[8],proj[9],proj[10],proj[11],
+      proj[12],proj[13],proj[14],proj[15]);
+  det=
+    proj[0]*proj[5]*proj[10] - proj[8]*proj[5]*proj[2]+
+    proj[1]*proj[6]*proj[8]  - proj[9]*proj[6]*proj[0]+
+    proj[2]*proj[4]*proj[9]  - proj[10]*proj[4]*proj[1];
+  printf("die Determinante der Drehmatrix ist: %g\n",det);*/
   mole.adp = 0;
   mole.intern = 0;
   mole.tubes = 0;
-  glDisable(GL_BLEND);
-  mole.atoms(xd, 50);
-  mole.bonds(bonds);
+  glDisable(GL_BLEND);  
+  glEnable(GL_DEPTH_TEST );
+  glDepthFunc(GL_LEQUAL);
+  mole.atoms(xd, 50, 0.5);//smaller atoms
+  mole.bonds(bonds,0.5);
   glEnable(GL_COLOR_MATERIAL);
   qglColor(dsrLabelColor);  // The fragment label color
-  glClear( GL_DEPTH_BUFFER_BIT);
+//  glClear( GL_DEPTH_BUFFER_BIT);
   for (int i=0; i<xd.size(); i++){
       GLdouble in[4], out[4];
       in[0] = xd.at(i).pos.x;
@@ -141,20 +193,32 @@ void DSRGlWindow::draw(){
       posTo2D(xd.at(i).pos,model,proj,viewport, &xd[i].screenX, &xd[i].screenY);
       bool issel=false;
       for (int j=0; j<selFragAt.size(); j++) {
-        if ((showFitLabel!=NULL)
+        if ((showFitLabel!=nullptr)
               && (!showFitLabel->isChecked())
               && (j<m_molecule->selectedatoms.size())
               && (xd.at(i).Label==selFragAt.at(j).Label)) {
           issel=true;
           renderText( xd.at(i).pos.x, xd.at(i).pos.y, xd.at(i).pos.z,
-              xd.at(i).Label+"->"+m_molecule->selectedatoms.at(j).Label, myFont);
+              xd.at(i).Label+"->"+m_molecule->selectedatoms.at(j).Label);
         }
       }
       if (!issel) {
         renderText( xd.at(i).pos.x, xd.at(i).pos.y, xd.at(i).pos.z,
-            xd.at(i).Label, myFont);
+            xd.at(i).Label);
       }
   }
+
+
+
+  for (int i=0; i<xd.size(); i++){
+      GLdouble in[4], out[4];
+      in[0] = xd.at(i).pos.x;
+      in[1] = xd.at(i).pos.y;
+      in[2] = xd.at(i).pos.z;
+      in[3] = 1.0;
+      transform_point(out, model, in);
+      posTo2D(xd.at(i).pos,model,proj,viewport, &xd[i].screenX, &xd[i].screenY);
+  }
   glPopMatrix();
   if (!selFragAt.isEmpty()){
     glPushMatrix();{
@@ -168,7 +232,7 @@ void DSRGlWindow::draw(){
       glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
     }glPopMatrix();
   }
-  if ((showFit!=NULL)&&(!fita.isEmpty())&&(showFit->isChecked())){
+  if ((showFit!=nullptr)&&(!fita.isEmpty())&&(showFit->isChecked())){
     glPushMatrix();{
       glEnable(GL_BLEND);
       glScaled( L, L, L );
@@ -182,9 +246,10 @@ void DSRGlWindow::draw(){
       if (showFitLabel->isChecked()){
         qglColor(QColor(0, 0, 175));  // The surrounding atoms label color
         for (int i=0; i<fita.size(); i++){
-          renderText( fita.at(i).pos.x, fita.at(i).pos.y, fita.at(i).pos.z, fita.at(i).Label, myFont);
+          renderText( fita.at(i).pos.x, fita.at(i).pos.y, fita.at(i).pos.z, fita.at(i).Label);
         }
       }
+
     }glPopMatrix();
   }
 }
@@ -222,19 +287,31 @@ void DSRGlWindow::initializeGL(){
   glMaterialf(  GL_FRONT_AND_BACK, GL_SHININESS,           OBJ_SHIN );
   glShadeModel( GL_SMOOTH );
   glEnable(GL_NORMALIZE);
-  glClearColor(0.6f,0.6f,0.6f,1.0f);
+  glClearColor(0.4f,0.4f,0.4f,1.0f);
   glEnable(GL_DEPTH_TEST );
   glDepthFunc(GL_LEQUAL);
   gluLookAt_(0.0, 200, 50 ,   0.0, 0.0, 0.0,   0.0, -100, 400 );
   glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
+
+  makeCurrent();
+  mole.g_Program=new QOpenGLShaderProgram(this);
+  mole.g_program=mole.installShader();
+  GLenum err = GL_NO_ERROR;
+  int ect=0;
+  printf("DSR shaderprog %d context is valid = %d \n",mole.g_program,context()->isValid());
+  if (!mole.g_program) printf("%s\n",mole.g_Program->log().toStdString().c_str());
+  while((err = glGetError()) != GL_NO_ERROR){
+    printf("in DSR: GL_INVALID_VALUE%d GL_INVALID_OPERATION%d [%d] err=%x\n",err==GL_INVALID_VALUE ,err == GL_INVALID_OPERATION,++ect,err);
+  }
 }
 
 void DSRGlWindow::resizeGL(int width, int height){
 #if defined Q_OS_MAC && (QT_VERSION >= 0x050000)
-  glViewport(0, 0, width*2,height*2);        
-#else  
-  glViewport(0, 0, width, height);        
+    retinafktr = 2;
+#else
+    retinafktr = 1;
 #endif
+  glViewport(0, 0, width*retinafktr, height*retinafktr);
   glGetIntegerv(GL_VIEWPORT, vp);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
@@ -255,14 +332,14 @@ void DSRGlWindow::paintGL(){
   glClearColor(0.6f,0.6f,0.6f,1.0f); // The background color
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 #if defined Q_OS_MAC && (QT_VERSION >= 0x050000)
-  glViewport(0, 0, QGLWidget::width()*2, QGLWidget::height()*2);        
+  glViewport(0, 0, QOpenGLWidget::width()*2, QOpenGLWidget::height()*2);
 #else  
-  glViewport(0, 0, QGLWidget::width(), QGLWidget::height());        
+  glViewport(0, 0, QOpenGLWidget::width(), QOpenGLWidget::height());
 #endif
   glGetIntegerv(GL_VIEWPORT, vp);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
-  gluPerspective_( 29.0, (double)QGLWidget::width()/QGLWidget::height(), 5.0, 8000.0 );
+  gluPerspective_( 29.0, (double)QOpenGLWidget::width()/QOpenGLWidget::height(), 5.0, 8000.0 );
   glMatrixMode(GL_MODELVIEW);
   glPushMatrix();
   draw();
@@ -277,7 +354,7 @@ void DSRGlWindow::clear_molecule()
   fitabonds.clear();
   selFragAt.clear();
   source_atoms.clear();
-  updateGL();
+  update();
 }
 
 void DSRGlWindow::display_fragment(DSRMol frag, bool clear_sel)
@@ -321,7 +398,7 @@ void DSRGlWindow::display_fragment(DSRMol frag, bool clear_sel)
   }
   L = 90.0/mole.dimension(xd); // Zoom level of the molecule inside the widget
   bonds=mole.connecting(xd,true);
-  updateGL();
+  update();
 }
 
 
@@ -334,7 +411,7 @@ void DSRGlWindow::set_label_color() {
     } else {
         dsrLabelColor = QColor("white");
     }
-    updateGL();
+    update();
 }
 
 
@@ -346,7 +423,7 @@ void DSRGlWindow::mousePressEvent(QMouseEvent *event){
   //! With more than three atoms, the first selection will be removed.
   moux=event->pos().x();
   mouy=event->pos().y();
-  if((event->buttons() == Qt::RightButton && event->modifiers() == Qt::ControlModifier)){
+  if((event->buttons() == Qt::RightButton) && (event->modifiers() == Qt::ControlModifier)){
       QMenu *menu = new QMenu("");
       menu->addAction(QString("Set label color"), this, SLOT(set_label_color()));
       menu->exec(QCursor::pos());
@@ -358,8 +435,8 @@ void DSRGlWindow::mousePressEvent(QMouseEvent *event){
     int nahdai=-1;
     for (int j=0; j<xd.size();j++){
       if (xd.at(j).hidden) continue;
-      da=(((xd.at(j).screenX-event->x())*( xd.at(j).screenX-event->x()))+
-          ((xd.at(j).screenY-event->y())*( xd.at(j).screenY-event->y())));
+      da=(((xd.at(j).screenX-event->pos().x())*( xd.at(j).screenX-event->pos().x()))+
+          ((xd.at(j).screenY-event->pos().y())*( xd.at(j).screenY-event->pos().y())));
       nahdai=(da<nahda)?j:nahdai;
       nahda=qMin(nahda,da);
     }
@@ -370,7 +447,7 @@ void DSRGlWindow::mousePressEvent(QMouseEvent *event){
       //}
       selFragAt.append(xd[nahdai]);
       if (selFragAt.size() > 3) selFragAt.removeFirst();
-      updateGL();
+      update();
       if (selFragAt.size() == 3) {
         source_atoms = QString("%1 %2 %3")
                         .arg(selFragAt.at(0).Label)
@@ -436,6 +513,7 @@ inline void DSRGlWindow::__RotateCS( double c, double s, double& X, double& Y )
 
 void DSRGlWindow::glTranslateL( const double dx, const double dy, const double dz ) {
   double mat[4][4];
+  makeCurrent();
 
   glGetDoublev( GL_MODELVIEW_MATRIX, (double*)mat );
   mat[3][0] += dx;  mat[3][1] += dy;  mat[3][2] += dz;
@@ -448,6 +526,7 @@ void DSRGlWindow::glRotateL( const double dang, const double x, const double y,
 #endif
 
   double s = z;
+  makeCurrent();
   s = sin(dang*M_PI/180);
   const double c = cos(dang*M_PI/180);
   glGetDoublev( GL_MODELVIEW_MATRIX, (double*)mat );
@@ -465,10 +544,10 @@ void DSRGlWindow::glRotateL( const double dang, const double x, const double y,
   }else{
     __RotateCS( c, s, mat[0][0], mat[0][1] );
     __RotateCS( c, s, mat[1][0], mat[1][1] );
-    __RotateCS( c, s, mat[2][0], mat[2][1] );
+    __RotateCS( c, s, mat[2][0], mat[2][1] );    
     //    printf("z\n");
   }
-  glLoadMatrixd((double*)mat);
+  glLoadMatrixd((double*)mat); 
 }
 
 
@@ -478,17 +557,20 @@ void DSRGlWindow::mouseMoveEvent(QMouseEvent *event){
   //! or zooms the molecule (right mouse button).
   double x = event->pos().x();
   double y = event->pos().y();
-  GLfloat dx = GLfloat( event->x() - moux) / width();
-  GLfloat dy = GLfloat( event->y() - mouy) / height();
+
+  GLfloat dx = GLfloat( event->pos().x() - moux) / width();
+  GLfloat dy = GLfloat( event->pos().y() - mouy) / height();
   if((event->buttons() & Qt::RightButton)){
+  //printf("mouseMoveEvent %f %f %f %f %x\n", x, y, moux, mouy, event->buttons());
+  makeCurrent();
     glScaled(1.0-dy,1.0-dy,1.0-dy);
-  }else if((event->buttons() && Qt::LeftButton)){
+  }else if((event->buttons() & Qt::LeftButton)){
     glRotateL(dy*360.0,1.0,0.0,0.0);
     glRotateL(dx*360.0,0.0,1.0,0.0);
   }
   moux=x;
   mouy=y;
-  updateGL();
+  update();
 } 
 
 void DSRGlWindow::wheelEvent(QWheelEvent *event){
@@ -502,14 +584,18 @@ Qt::MetaModifier	      0x10000000  A Meta key on the keyboard is pressed.
 Qt::KeypadModifier	    0x20000000  A keypad button is pressed.
 Qt::GroupSwitchModifier	0x40000000  X11 only. A Mode_switch key on the keyboard is pressed.
 */ 
-  int numDegrees = event->delta() / 8;
+#if (QT_VERSION>=0x050000)
+    int numDegrees = event->angleDelta().y() / 8;
+#else
+    int numDegrees = event->delta() / 8;
+#endif
   int numSteps = numDegrees / 15;
-  if (event->modifiers()==Qt::NoModifier){
+  if (event->modifiers()==Qt::NoModifier){       
     int d = myFont.pointSize();
     d = (d+numSteps>4)?d+numSteps:d;
     d = qMax(d,7);
     myFont.setPointSize(d);
-    updateGL();
+    update();
   }
 
 }
@@ -518,7 +604,7 @@ void DSRGlWindow::tryfit(){
   if (showFit==NULL) return;
   fita.clear();
   fitabonds.clear();
-  if ((selFragAt.size()!=3)||(m_molecule->selectedatoms.size()!=3)) {updateGL();return;}
+  if ((selFragAt.size()!=3)||(m_molecule->selectedatoms.size()!=3)) {update();return;}
   
   //tree orthogonal vectors of unit size form a rotation matrix 
   //the transponse of the latter is its inverse
@@ -536,7 +622,7 @@ void DSRGlWindow::tryfit(){
   b2=b3%b1;
   Matrix bmat=transponse(Matrix(b1,b2,b3));
   if ( (a1*(a2%a3)<0.9) || (b1*(b2%b3)<0.9) ) {  //linear dependence id determinant is 0
-    updateGL();
+    update();
     return;
   }
   // m_molecule->showatoms was m_molecule->asymm before, but showatoms is needed for disorder
@@ -553,8 +639,8 @@ void DSRGlWindow::tryfit(){
     }
   }  
   fitabonds=mole.connecting(fita,true);
-  updateGL();
-  printf("FIT:\n================================================================================\n");
+  update();
+//  printf("FIT:\n================================================================================\n");
   for (int i=0; i<xd.size(); i++){
     int imin=0;
     double dmin=10.0;
@@ -569,7 +655,10 @@ void DSRGlWindow::tryfit(){
       if (d<dmin) imin=j;
       dmin=(d<dmin)?d:dmin;
     }
-    printf("%-8s <==> %-8s %f\n", xd.at(i).Label.toStdString().c_str(),
-           m_molecule->showatoms.at(imin).Label.toStdString().c_str(), sqrt(dmin));
+//    printf("%-8s <==> %-8s %f\n", xd.at(i).Label.toStdString().c_str(),
+//           m_molecule->showatoms.at(imin).Label.toStdString().c_str(), sqrt(dmin));
   }
 }
+void DSRGlWindow::qglColor(const QColor color) {
+    glColor4d(color.redF(), color.greenF(), color.blueF(), color.alphaF());
+}
