@@ -60,7 +60,8 @@ DSRGui::DSRGui(Molecule *mole, QString shelxlPath, Window *parent): QWidget(pare
   m_shelxle = parent;
   m_molecule = mole;
   m_shelxlPath = shelxlPath;
-  this->setWindowFlags(Qt::Window);
+  this->setWindowFlags(Qt::Window|Qt::WindowStaysOnTopHint);
+  //printf("I am DSR and this are my flags:%d\n",windowFlags ());
   setWindowTitle(QString(tr("DSR GUI")));
   if (mysystem == QString("win")) {
     dsrpath = getDSRDir()+"/dsr.bat";
@@ -81,22 +82,22 @@ DSRGui::DSRGui(Molecule *mole, QString shelxlPath, Window *parent): QWidget(pare
   fragmentsList = new QVector<QStringList>;
   fragmentNameTag.clear();
   header = new DSRMol;
-  mainVLayout = new QVBoxLayout(this);
-  editLayout = new QHBoxLayout;
-  chooserLayout = new QGridLayout;
-  searchLayout = new QHBoxLayout;
-  partLayout = new QHBoxLayout;
-  fvarLayout = new QHBoxLayout;
-  occLayout = new QHBoxLayout;
-  resclassLayout = new QHBoxLayout;
-  optionsLayout1 = new QVBoxLayout;
-  optionsLayout2 = new QVBoxLayout;
-  optionsLayout3 = new QVBoxLayout;
+  mainLayout = new QGridLayout(this);
+  editLayout = new QHBoxLayout();
+  //chooserLayout = new QGridLayout;
+  searchLayout = new QHBoxLayout();
+  //partLayout = new QHBoxLayout();
+  //fvarLayout = new QHBoxLayout();
+  //occLayout = new QHBoxLayout();
+  resclassLayout = new QHBoxLayout();
+  optionsLayout2 = new QVBoxLayout();
+  optionsLayout3 = new QVBoxLayout();
   groupBox1 = new QGroupBox();
+  optionsLayout1 = new QGridLayout(groupBox1);
   groupBox2 = new QGroupBox();
   residueOptionsBox = new QGroupBox(tr("Use a Residue"));
-  buttonLayout = new QVBoxLayout;
-  optionboxes = new QHBoxLayout;
+  buttonLayout = new QVBoxLayout();
+  optionboxes = new QHBoxLayout();
   fitDSRButton = new QPushButton(tr("Fit Fragment!"));
   splitButton = new QPushButton(tr("Split Atoms"));
   fitDSRButton->setStyleSheet("QPushButton {"
@@ -118,7 +119,8 @@ DSRGui::DSRGui(Molecule *mole, QString shelxlPath, Window *parent): QWidget(pare
   SearchInp = new QLineEdit;//(tr("Search a fragment here"));
   partLabel = new QLabel(tr("PART"));
   fvarLabel = new QPushButton(tr("Free Variable"));  // Is a QPushbutton to have a clicked() signal
-  fvarLabel->setStyleSheet("QPushButton {border-style: outset; border-width: 0px;}");  // remove border to appear like a label
+  fvarLabel->setStyleSheet("QPushButton {border-style: outset; border-width: 0px; text-align:left;}");  // remove border to appear like a label
+  fvarLabel->setToolTip("Click here to invert the current free variable");
   occLabel = new QLabel(tr("Occupancy"));
   classLabel = new QLabel(tr("Residue Class"));
   usermanualLabel = new QLabel(tr("<a href=\"https://github.com/dkratzert/DSR/raw/master/manuals/DSR-manual.pdf\"> DSR User Manual </a>"));
@@ -130,8 +132,8 @@ DSRGui::DSRGui(Molecule *mole, QString shelxlPath, Window *parent): QWidget(pare
   outtext->setOpenExternalLinks(true);
   info = new QTextBrowser;
   //this->move(-this->width(), -this->height());
+  info->setMinimumWidth(235);  
   info->hide();
-  info->setMinimumWidth(235);
   //info->setMaximumWidth(235);
   fragmentTableView = new QTableView;
   occEdit = new QLineEdit;
@@ -140,16 +142,15 @@ DSRGui::DSRGui(Molecule *mole, QString shelxlPath, Window *parent): QWidget(pare
   resiclassEdit = new QLineEdit;
   // layout for the interactions
   optionboxes->addWidget(groupBox1);
-  optionboxes->addStretch();
+  //optionboxes->addStretch();
   optionboxes->addWidget(groupBox2);
-  optionboxes->addStretch();
+  //optionboxes->addStretch();
   optionboxes->addWidget(residueOptionsBox);
-  optionboxes->addStretch();
+  //optionboxes->addStretch();
   optionboxes->addLayout(buttonLayout);
-  buttonLayout->setSizeConstraint(QLayout::SetMaximumSize);
+  //buttonLayout->setSizeConstraint(QLayout::SetMaximumSize);
   // The search field:
-  searchLayout->addWidget(searchLabel);
-  searchLayout->addWidget(SearchInp);
+
   SearchInp->setFocus();  // searching should be the default
   SearchInp->setMinimumWidth(getCharWidth(9));
   fragmentNameTag.clear();
@@ -161,18 +162,7 @@ DSRGui::DSRGui(Molecule *mole, QString shelxlPath, Window *parent): QWidget(pare
   mygl->showFitLabel = new QCheckBox(tr("labels on target overlay"));
   mygl->showFitLabel->setChecked(true);
   // Table of fragments and 3D window:
-  chooserLayout->addWidget(fragmentTableView,  0, 0, 1, 1);
-  chooserLayout->addWidget(mygl,               0, 1, 1, 2);
-  chooserLayout->addWidget(mygl->showFit,      2, 1, 1, 1);
-  chooserLayout->addWidget(mygl->showFitLabel, 2, 2, 1, 1);
-  chooserLayout->addLayout(searchLayout,       2, 0, 1, 1);
-  chooserLayout->setColumnStretch(0, 4);
-  chooserLayout->setColumnStretch(1, 3);
-  chooserLayout->setRowStretch(2, 0);
-  // Three main layours:
-  mainVLayout->addLayout(chooserLayout, 3);
-  mainVLayout->addLayout(editLayout, 2);
-  mainVLayout->addLayout(optionboxes, 0);
+
   editLayout->addWidget(outtext);
   editLayout->addWidget(info);
   editLayout->setStretchFactor(outtext, 2);
@@ -182,37 +172,34 @@ DSRGui::DSRGui(Molecule *mole, QString shelxlPath, Window *parent): QWidget(pare
   font.setStyleHint(QFont::TypeWriter);
   outtext->setFont(font);
   outtext->setMinimumHeight(170);
-  // Layouts for optionbox1
-  partLayout->addWidget(partspinner);
-  partLayout->addWidget(partLabel);
-  partLayout->addStretch();
   partspinner->setRange(-99, 99);
   partspinner->setValue(0);
-  fvarLayout->addWidget(fvarspinner);
-  fvarLayout->addWidget(fvarLabel);
-  fvarLayout->addStretch();
   fvarspinner->setRange(-99, 99);
   fvarspinner->setValue(1);
-  occLayout->addWidget(occEdit);
-  occLayout->addWidget(occLabel);
   occEdit->setValidator(new QDoubleValidator(0, 99, 5, occEdit));
   occEdit->setMaximumWidth(getCharWidth(5));
   occEdit->setMinimumWidth(getCharWidth(5));
   // box1
-  optionsLayout1->addLayout(partLayout);
-  optionsLayout1->addLayout(fvarLayout);
-  optionsLayout1->addLayout(occLayout);
-  optionsLayout1->addWidget(replaceModeBox);
-  optionsLayout1->addWidget(splitModeBox);
+  optionsLayout1->addWidget(partspinner,0,0);
+  optionsLayout1->addWidget(partLabel,0,1);
+
+  optionsLayout1->addWidget(fvarspinner,1,0);
+  optionsLayout1->addWidget(fvarLabel  ,1,1);
+
+  optionsLayout1->addWidget(occEdit ,2,0);
+  optionsLayout1->addWidget(occLabel,2,1);
+
+  optionsLayout1->addWidget(replaceModeBox, 3,0,1,2  );
+  optionsLayout1->addWidget(splitModeBox,   4,0,1,2  );
   splitModeBox->hide();
-  optionsLayout1->addStretch();
+  //optionsLayout1->addStretch();
   groupBox1->setLayout(optionsLayout1);
   // box2
   optionsLayout2->addWidget(invertFragBox);
   optionsLayout2->addWidget(runExtBox);
   optionsLayout2->addWidget(dfixBox);
   optionsLayout2->addWidget(rigidBox);
-  optionsLayout2->addStretch();
+  //optionsLayout2->addStretch();
   groupBox2->setLayout(optionsLayout2);
   // box3
   resclassLayout->addWidget(resiclassEdit);
@@ -243,12 +230,32 @@ DSRGui::DSRGui(Molecule *mole, QString shelxlPath, Window *parent): QWidget(pare
   painter.setFont(QFont("Sans-Serif", 12));
   painter.drawRect(rectangle);
   painter.drawText(QPoint(12, 30), "Loading fragment list...");
+  QGroupBox *glGroupBox = new QGroupBox();
+  glo = new QGridLayout(glGroupBox);
+  glo->addWidget(mygl,0,0,1,2);
+  glo->addWidget(mygl->showFit,1,0,1,1);
+  glo->addWidget(mygl->showFitLabel,1,1,1,1);
+
+  glGroupBox->setFlat(true);
+  QGroupBox *fGroupBox = new QGroupBox();
+  flo = new QGridLayout(fGroupBox);
+  mainLayout->addWidget(fGroupBox,          0, 0, 3, 2);
+  mainLayout->addWidget(glGroupBox,         0, 2, 3, 5);
+  flo->addWidget(fragmentTableView ,0,0,1,2);
+  flo->addWidget(searchLabel ,1,0,1,1);
+  flo->addWidget(SearchInp   ,1,1,1,1);
+  mainLayout->setRowMinimumHeight(3,200);
+  mainLayout->addLayout(editLayout,  3,0,1,7);
+  mainLayout->addLayout(optionboxes, 6,0,1,7);
   QSplashScreen *splash = new QSplashScreen(pix);
   splash->show();
   splash->showMessage("");
   target_atoms = getSelectedAtomsList(); // must be before signals
   // request version.txt to warn for updates:
+
+#if !defined(Q_OS_MAC) && (QT_VERSION >= 0x050000)
   getVersionFromServer();
+#endif  
   // Signal slot connections:
   connect_signals_and_slots();
   splitDecide();
@@ -261,18 +268,28 @@ DSRGui::DSRGui(Molecule *mole, QString shelxlPath, Window *parent): QWidget(pare
   // Has to be here, otherwise fragmentTableView is not initialized:
   QItemSelectionModel *sm = fragmentTableView->selectionModel();
   this->show();
-  if (!(sm == NULL)){  // prevents error about missing connection if DSR is not present
+  if (!(sm == nullptr)){  // prevents error about missing connection if DSR is not present
     connect(sm, SIGNAL(currentRowChanged(QModelIndex,QModelIndex)),
         this, SLOT(setFragName(QModelIndex)), Qt::UniqueConnection);
   }
-  this->move(QApplication::desktop()->width()-this->width()-30, 30);
-  this->resize(this->width()-2, this->height()-2);  // a trick to redraw the gl windows
+  //int scn=QApplication::desktop()->screenNumber(m_shelxle);//find out where the main window is
+  QScreen *screen = m_shelxle->window()->windowHandle()->screen();
+  //QGuiApplication::screens().at(scn)->geometry()
+  this->move(screen->geometry().x()+
+             screen->geometry().width()-this->width()-30, 30);
+
+  //printf("test\n");
+  //this->resize(this->width()-2, this->height()-2);  // a trick to redraw the gl windows
+  // this raises an 'Unable to set geometry' error
+  mygl->update();
+  //printf("test2\n");
   if (!(dsrVersion.toLatin1() == "0")) {
     outtext->append(QString("\nFound DSR version %1.").arg(dsrVersion));
     if (dsrVersion.toInt() > 9999) {  // To be changed later
         splitButton->show();  // logic turned, because a non-replying web server turned this button on
     }
   }
+  update();
 }
 
 
@@ -327,7 +344,7 @@ void DSRGui::connect_signals_and_slots() {
             this, SLOT(searchFragment(QString)));
     connect(occEdit, SIGNAL(textChanged(QString)),
             this, SLOT(setFvarOcc(void)));
-    connect(fvarspinner, SIGNAL(valueChanged(QString)),
+    connect(fvarspinner, SIGNAL(valueChanged(int)),
             this, SLOT(setFvarOcc(void)));
     connect(resiclassEdit, SIGNAL(textChanged(QString)),
             this, SLOT(setResiClass(QString)));
@@ -353,23 +370,29 @@ void DSRGui::connect_signals_and_slots() {
     connect(this, SIGNAL(currentFragmentChanged(DSRMol)),
             mygl, SLOT(display_fragment(DSRMol)));
     connect(m_shelxle->chgl, SIGNAL(selectionChanged()),
-            mygl, SLOT(updateGL()));
+            mygl, SLOT(update()));
     connect(m_shelxle->chgl, SIGNAL(selectionChanged()),
             this, SLOT(updateTarget()));
     connect(m_shelxle->chgl, SIGNAL(selectionChanged()),
             this, SLOT(update()));
     connect(mygl->showFitLabel, SIGNAL(stateChanged(int)),
-            mygl, SLOT(updateGL())); 
+            mygl, SLOT(update()));
+    connect(mygl->showFitLabel, SIGNAL(stateChanged(int)),
+            mygl, SLOT(update()));
     connect(mygl->showFit, SIGNAL(stateChanged(int)),
-            mygl, SLOT(updateGL()));
+            mygl, SLOT(update()));
+    connect(mygl->showFit, SIGNAL(stateChanged(int)),
+            mygl, SLOT(update()));
     connect(m_shelxle->chgl, SIGNAL(selectionChanged()),
             mygl, SLOT(makeInfo()));
     connect(mygl, SIGNAL(sourceStringChanged()),
             this, SLOT(combineOptionstext()));
     connect(mygl, SIGNAL(updateInfo(QString)),
             this, SLOT(setInfo(QString)));
+#if !defined(Q_OS_MAC) && (QT_VERSION >= 0x050000)
     connect(net_manager, SIGNAL(finished(QNetworkReply*)),
             this, SLOT(isDSRUpToDate(QNetworkReply*)));
+#endif 
     connect(fvarLabel, SIGNAL(clicked(bool)),
             this, SLOT(invertFvar()));
     connect(m_shelxle->chgl, SIGNAL(selectionChanged()),
@@ -395,6 +418,7 @@ void DSRGui::getVersionFromServer() {
     reply = net_manager->get(request);
   }
 
+/*
 bool DSRGui::runTCPServer() {
   //! Socket server to get messages from DSR. Might be used in future.
   tcpServer = new QTcpServer(this);
@@ -411,7 +435,8 @@ bool DSRGui::runTCPServer() {
   }
   return true;
 }
-
+*/
+/*
 void DSRGui::ReadClientData() {
   //! Reads DSR client data from a tcp soccet connection.
   QByteArray data_buffer;
@@ -427,6 +452,7 @@ void DSRGui::ReadClientData() {
   outtext->append(data_buffer);
   //dsrResulttext.append(data_buffer);
 }
+*/
 
 void DSRGui::isDSRUpToDate(QNetworkReply* reply) {
   /*
@@ -517,13 +543,13 @@ QStringList DSRGui::which(QString programName) {
 #else
     pathList = QString(qgetenv("PATH")).split(":");
 #endif
+    pathList.append("/usr/local/bin/");
   }
   execlist << ".bat" << ".exe" << "";
   foreach (QString exec, execlist) {
     foreach (QString path, pathList) {
       QString fullpath = path+"/"+programName+exec;
-      QFileInfo fi;
-      fi = fullpath;
+      QFileInfo fi(fullpath);
       if (QFile::exists(fullpath) && (fi.isExecutable())){
         foundInPath.append(QDir::cleanPath(fullpath));
       }
@@ -626,7 +652,7 @@ bool DSRGui::open_last_fileversion() {
     }
     QString savehist_content = savehist.readAll();  //Entry|@|2011-03-08T13:32:19|@|b11.res|@|
     savehist.close();
-    QRegExp re = QRegExp("Entry\\|@\\|\\w+-\\w+-\\w+:\\w+:\\w+\\|@\\|[^@]+@\\|");
+    QRegularExpression re = QRegularExpression("Entry\\|@\\|\\w+-\\w+-\\w+:\\w+:\\w+\\|@\\|[^@]+@\\|");
     QStringList entries = savehist_content.split(re, skipEmptyParts);
     QFile f(m_shelxle->dirName);
     bool success = f.open(QIODevice::WriteOnly|QIODevice::Text);
@@ -668,7 +694,7 @@ void DSRGui::setInfo(QString s) {
 
 void DSRGui::splitDecide(void) {
     //! Enables the "split atoms" button if at least one atom is selected.
-    if (target_atoms.count() >= 1) {
+    if (target_atoms.length() >= 1) {
         splitButton->setEnabled(true);
     }
 }
@@ -708,7 +734,7 @@ int DSRGui::findFVARlines(QStringList *reslist) {
   int fvarline;
   // I need lastIndexOf() here, because in case of
   // several FVAR lines indexOf() would fail:
-  fvarline = reslist->lastIndexOf(QRegExp("^FVAR.*", Qt::CaseInsensitive));
+  fvarline = reslist->lastIndexOf(QRegularExpression("^FVAR.*", QRegularExpression::CaseInsensitiveOption));
   if (fvarline > 0) {
     return fvarline;
   } else {
@@ -722,7 +748,7 @@ QVector<int> DSRGui::findDSRLines(QStringList *reslist) {
   QVector<int> lineNums;
   int Num = 0;
   foreach (QString line, *reslist) {
-    if (line.contains(QRegExp("^rem\\s{1,6}DSR\\s{1,6}.*", Qt::CaseInsensitive))) {
+    if (line.contains(QRegularExpression("^rem\\s{1,6}DSR\\s{1,6}.*", QRegularExpression::CaseInsensitiveOption))) {
       lineNums.append(Num);
     }
     Num++;
@@ -777,7 +803,7 @@ QString DSRGui::findFreeResiNumber() {
       resiset.insert(atom.resiNr);
     }
   }
-  resilist = resiset.toList();
+  resilist = resiset.values();//.toList();
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
   std::sort(resilist.begin(), resilist.end());
 #else
@@ -859,7 +885,7 @@ bool DSRGui::getFragmentHeader(QString frag) {
     //editButton->setDisabled(true); // editing the empty header would crash ShelXle
     return false;
   }
-  headerlist->append(rawheader->split(QRegExp("\n|\r\n|\r")));
+  headerlist->append(rawheader->split(QRegularExpression("\n|\r\n|\r")));
   QStringList line;
   for (int i=0; i<headerlist->size(); i++) {
     if (headerlist->at(i).size() == 0){
@@ -1018,6 +1044,9 @@ void DSRGui::combineOptionstext(void) {
                           target_atoms+" "+part+" "+fvarocc+" "+dfix+" "+resistr+split);
   combiDSRline = outstring.simplified().toUpper();
   outtext->append(combiDSRline);
+
+  mygl->selchanged = true;
+  //printf("combineOptionstext\n");
 }
 
 
@@ -1134,12 +1163,15 @@ void DSRGui::invertFrag(bool checked) {
   } else {
     this->invert = false;
   }
+  /*
   V3 v1 = V3(-1,  0,  0);
   V3 v2 = V3( 0, -1,  0);
   V3 v3 = V3( 0,  0,  1);
   //rotation matrix to rotate the selection 180deg.
   // for less disturbance of the inverted view:
-  Matrix rmat = Matrix(v1, v2, v3);
+  // ch: WHY??
+
+  Matrix rmat = Matrix(v1, v2, v3);*/
   // Invert the OpenGL coordinates:
   double coord;
   int index;
@@ -1147,26 +1179,20 @@ void DSRGui::invertFrag(bool checked) {
   foreach(QStringList line, header->atoms) {
     for (int n = 2; n<5; n++) {
       coord = line[n].toDouble();
-      // rotate fragment 180 deg.
-      if ((n == 2) | (n == 3)) {
-        coord = coord * -1;
-      }
+      // rotate fragment 180 deg. if ((n == 2) || (n == 3)) { coord = coord * -1; }
       header->atoms[index][n] = QString("%1").arg(-coord);
     }
     index = index+1;
   }
   //Also invert the selection of the atoms:
   for (int n=0; n<mygl->selFragAt.size(); n++) {
-    //this line is essential:
-    mygl->selFragAt[n].pos = mygl->selFragAt[n].pos * rmat;
-    mygl->selFragAt[n].pos.x *= -1;
-    mygl->selFragAt[n].pos.y *= -1;
-    mygl->selFragAt[n].pos.z *= -1;
+    //this line is essential:    mygl->selFragAt[n].pos = mygl->selFragAt[n].pos * rmat;
+    mygl->selFragAt[n].pos *= -1.0;
   }
   mygl->display_fragment(*header, false);
-  // This mouse press hack reactivates the fit overlay after inversion:
-  QMouseEvent ev ((QEvent::MouseButtonPress), QPoint(0, 0), Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
-  mygl->mousePressEvent(&ev);
+  combineOptionstext();
+  mygl->makeInfo();
+  mygl->update();
 }
 
 void DSRGui::rigid_group(bool checked)
@@ -1225,7 +1251,7 @@ QString DSRGui::getSelectedAtomsList() {
   QStringList atoms;
   atoms.clear();
   for (int i=0; i<m_molecule->selectedatoms.size(); i++) {
-    atoms.append(m_molecule->selectedatoms.at(i).Label.toLatin1().replace(QChar(187), ">>"));
+    atoms.append(QString(m_molecule->selectedatoms.at(i).Label.toLatin1()).replace(QChar(187), ">>"));
   }
   return atoms.join(" ");
 }
@@ -1446,7 +1472,7 @@ void DSRGui::searchFragment(QString searchName) {
   this->runDSR(option, false);
   QVector<QStringList> fraglist;
   fraglist.clear();
-  foreach (QString line, dsrResulttext.split(QRegExp("\n|\r\n|\r"))) {
+  foreach (QString line, dsrResulttext.split(QRegularExpression("\n|\r\n|\r"))) {
     if (line.isEmpty()){
       continue;
     }
@@ -1483,7 +1509,7 @@ bool DSRGui::listDSRdbFragments(QString fav="None") {
   fragmentNameTag.clear();
   QString options = " -lc";
   runDSR(options, true);
-  QStringList frag_str_list = dsrResulttext.split(QRegExp("\n|\r\n|\r"));
+  QStringList frag_str_list = dsrResulttext.split(QRegularExpression("\n|\r\n|\r"));
   bool versionOk = false;
   foreach (QString line, frag_str_list) {
     if (line.contains("Duplicate database entry")) {
