@@ -19,6 +19,14 @@
 #ifndef EACDLG_H
 #define EACDLG_H
 #include <QtGui>
+#include <QOpenGLWidget>
+#include <QCheckBox>
+#include <QComboBox>
+#include <QDialog>
+#include <QSpinBox>
+#include <QGridLayout>
+//#include <>
+#include <QScrollArea>
 #include "molecule.h"
 #include "psewidget.h"
 /*! \brief EacGLW is the QGLWidget of the Edit Atom Styles Dialog EacDlg. 
@@ -29,23 +37,39 @@
  * setting which can be changed in the EacDlg. 3 Q-Peaks are displayed arround the molecule. 
  */
 
-class EacGLW: public QGLWidget {
+class EacGLW: public QOpenGLWidget {
 Q_OBJECT
   public:
+    explicit EacGLW( Molecule *externMole,QWidget * parent = 0);
 
-  QGLShaderProgram *saveProgram;
+  QOpenGLShaderProgram *saveProgram;
   Molecule *mol;//!< pointer to ShelXle's Molecule object.
   double exmin,//!< Minimal Q-Peak electron density value of shelXle to be restored when the dialog is closed.
 	 exmax;//!< Maximal Q-Peak electron density value of shelXle to be restored when the dialog is closed.
   CEnvironment xd;//!< MyAtom list of the molecule
   Connection bonds;//!< MyBond list of the molecule
   int proba;//!< probability level of the ellipsoids.
-  EacGLW( Molecule *externMole,QWidget * parent = 0);
   //EacGLW(const MyAtom *atom,Molecule *externMole);
   QFont myFont;//!< a font for the atom labels
   //QStringList pse;
+  PSEWidget *psew;  
+  QSettings *einstellung;
   QTimer *timer;//!< a timer for the molecule rotation
+  void qglColor(const QColor c);
+  void renderText(double x, double y, double z, const QString &str);
   void setMSAA(bool);
+  inline QString glError2String(GLenum err){
+        switch (err) {
+        case GL_NO_ERROR: return QString("GL_NO_ERROR: No OpenGL error."); break;
+        case GL_INVALID_ENUM: return QString("GL_INVALID_ENUM: An enumeration parameter is not a legal enumeration for that function."); break;
+        case GL_INVALID_VALUE: return QString("GL_INVALID_VALUE: Given when a value parameter is not a legal value for that function."); break;
+        case GL_INVALID_OPERATION: return QString("GL_INVALID_OPERATION: Given when the set of state for a command is not legal for the parameters given to that command."); break;
+        case GL_STACK_OVERFLOW: return QString("GL_STACK_OVERFLOW: Given when a stack pushing operation cannot be done because it would overflow the limit of that stack's size."); break;
+        case GL_STACK_UNDERFLOW: return QString("GL_STACK_UNDERFLOW: Given when a stack popping operation cannot be done because the stack is already at its lowest point."); break;
+        case GL_OUT_OF_MEMORY: return QString("GL_OUT_OF_MEMORY: Given when performing an operation that can allocate memory, and the memory cannot be allocated."); break;
+        default : return QString("Some other OpenGL error."); break;
+        }
+    }
   signals:
       void colorchanged(QColor bgcolor);//!< colorchanged is emitted  by the color dialog colorDLG to change the color of a chemical element. @param bgcolor the new color
  protected:
@@ -53,7 +77,7 @@ Q_OBJECT
   void resizeGL(int width, int height);
   void paintGL();
  public slots:
-  void changeAtom(int OZ);//!< changes the atom type of the central atom in OZ @param OZ atomic number (staring from zero) of the new atom
+  void changeAtom(int OZ, bool HisD);//!< changes the atom type of the central atom in OZ @param OZ atomic number (staring from zero) of the new atom
   void colorDLG();//!< changes the color of an atom type
   void wallA(int st);//!< wallA changes the state of the ellipsoid wall. @param st is the checkbox state of the calling QCheckBox.
   void ringA(int st);//!< ringA changes the state of the principal ellipses. @param st is the checkbox state of the calling QCheckBox.
@@ -65,6 +89,13 @@ Q_OBJECT
  private:
   GLint vp[4];
   void draw();
+  int retinafktr = 1;
+  void project(double x, double y, double z, GLdouble &tx, GLdouble &ty);
+  static inline void transform_point(GLdouble out[4], const GLdouble m[16], const GLdouble in[4]); 
+  static inline bool posTo2D(V3 obj, const GLdouble model[16], const GLdouble proj[16],
+    const GLint viewport[4], GLdouble &winx, GLdouble &winy,int retinafktr); 
+  QColor backGroundColor;
+  QColor labelColor;
 };
 /*! \brief MultiStyle a small dialog called in EacDlg to change the style of multiple elements at once.
  */
@@ -93,25 +124,25 @@ Q_OBJECT
 class EacDlg: public QDialog {
 Q_OBJECT
   public:
-  EacDlg(Molecule *externMole);
+  explicit EacDlg(Molecule *externMole);
   //EacDlg(MyAtom *onlyThisAtom,Molecule *externMole);
-  EacGLW *eacGLW;//!<The QGLWidget displaying an example molecule.
   public slots:
   void colorDLG2();//!< opens the bond color dialog.
   void colorDLGenvibon();
   void colorDLGenvidef();
   void colorDLGenvihbo();
   void colorchange(QColor bgcolor);//!< changes the color of the current element to bgcolor. Changes the color of the corresponding button.
-  void updateLEs(int OZ);//!< when the current element changes the settings of this element are loaded and applied to the EacGLW widget and the PSEWidget. @param OZ atomic number (staring from zero) of the new atom
+  void updateLEs(int OZ, bool HisD);//!< when the current element changes the settings of this element are loaded and applied to the EacGLW widget and the PSEWidget. @param OZ atomic number (staring from zero) of the new atom
   void checkWall();//!< when solid is checked then solid is unchecked.
-  void checkShader(bool b);//!< when solid is checked then solid is unchecked.
+  void checkShader(bool b);
   void checkSolid();//!< when walls are checked solid is unchecked. 
   void checkBall();//!< when ellipsoid surface is unchecked the solid checkbox is diabled. 
   void doMulti();//!< calls MultiStyle dialog
   void unicolor(bool b);//!< the bonds shall have a uniq color. 
   void woADP(bool b);//!< the elemen shall be shown without ADP ellispoids.
   void probaChanged(int ix);//!< The ellipsoid probability is changed to ix.
-  void strengthChanged(double d);//!< the bond thicknes is changed to d.
+  void strengthChanged(double d);//!< the bond thicknes is changed to d.  
+  void qStrengthChanged(double d);//!< the bond thicknes is changed to d.
   void QPeakRadChanged(double d);//!< QPeak size is changed to d.
   void lodChanged(int d);//!< the level of detail is changed to d.
   void kradChanged(double d);//!< the colvalent radius is changed to d.   
@@ -125,11 +156,12 @@ Q_OBJECT
   void accept () ;
   void reject () ;
  private:
+  EacGLW *eacGLW;//!<The QGLWidget displaying an example molecule.
   Molecule *mol;
   //MyAtom *otm;
-
-  QCheckBox *shaderUse; 
+  QCheckBox *shaderUse;
   QDoubleSpinBox *bondStrength;
+  QDoubleSpinBox *q_bondWidth;
   QSpinBox *lodSpinBox;
   QComboBox *eacProba;
   PSEWidget *psew;
