@@ -1,9 +1,16 @@
 #include "fcvsfo.h"
 #include "deprecation.h"
+#include "itsme.h"
 
 FcVsFo::FcVsFo(QString filename,double a,double b, double c, double al, double be, double ga, double lambda){
   const double g2r=180.0/M_PI;
-  cell.wave=lambda;
+  QString theIconPath;
+  if (!QString(PROGRAM_NAME).compare("shelXle")){
+      theIconPath=QString(":/xle-icons/");
+  }else{
+      theIconPath=QString(":/bicons/");
+  }
+  cell.wave=lambda;  
   cell.a=a;
   cell.b=b;
   cell.c=c;
@@ -91,10 +98,10 @@ FcVsFo::FcVsFo(QString filename,double a,double b, double c, double al, double b
   txt2->moveBy(480,503); 
   txt2->setData(0,-1);
   QToolBar *tb=new QToolBar(this);
-  tb-> addAction(QIcon(":/xle-icons/zoom-in.png"),"Zoom +",this,SLOT(zoomin()));
-  tb-> addAction(QIcon(":/xle-icons/zoom-out.png"),"Zoom -",this,SLOT(zoomout()));
-  tb-> addAction(QIcon(":/xle-icons/zoom-fit.png"),"Zoom selected",this,SLOT(zoomfit()));
-  tb-> addAction(QIcon(":/xle-icons/zoom-off.png"),"Zoom out",this,SLOT(zoomoff()));
+  tb-> addAction(QIcon(theIconPath+"zoom-in.svg"),"Zoom +",this,SLOT(zoomin()));
+  tb-> addAction(QIcon(theIconPath+"zoom-out.svg"),"Zoom -",this,SLOT(zoomout()));
+  tb-> addAction(QIcon(theIconPath+"zoom-fit.svg"),"Zoom selected",this,SLOT(zoomfit()));
+  tb-> addAction(QIcon(theIconPath+"zoom-off.svg"),"Zoom out",this,SLOT(zoomoff()));
   QComboBox *cbx=new QComboBox(this);
   cbx->addItem("Fo vs. Fc",1);
   cbx->addItem("Delta( F^2)/sigma vs. sin(theta/lambda)");
@@ -174,9 +181,9 @@ void FcVsFo::readFCF(){
   QString test=QString(f.readLine(120));
   QStringList lines;
   if (test.contains("index")) listType=1;
-  //qDebug()<<test<<test.contains(QRegExp("\\s*\\d+\\s*\\d+\\s*\\d+\\s*\\d+\\.\\d+\\s*\\d+\\.\\d+\\s*\\d+"));
-  if (test.contains(QRegExp("\\s*\\d+\\s*\\d+\\s*\\d+\\s*\\d+\\.\\d+\\s*\\d+\\.\\d+\\s*\\d+"))) listType=2;
-  if (test.contains(QRegExp("\\s*\\d+\\s*\\d+\\s*\\d+\\s*\\d+\\.\\d+\\s*\\d+\\.\\d+\\s*\\d+\\.\\d+"))) listType=5;
+  //qDebug()<<test<<test.contains(QRegularExpression("\\s*\\d+\\s*\\d+\\s*\\d+\\s*\\d+\\.\\d+\\s*\\d+\\.\\d+\\s*\\d+"));
+  if (test.contains(QRegularExpression("\\s*\\d+\\s*\\d+\\s*\\d+\\s*\\d+\\.\\d+\\s*\\d+\\.\\d+\\s*\\d+"))) listType=2;
+  if (test.contains(QRegularExpression("\\s*\\d+\\s*\\d+\\s*\\d+\\s*\\d+\\.\\d+\\s*\\d+\\.\\d+\\s*\\d+\\.\\d+"))) listType=5;
   if ((test.startsWith("#"))||(test.startsWith("data"))){
     while (!test.isEmpty()){
       test=QString(f.readLine(120));
@@ -199,7 +206,7 @@ void FcVsFo::readFCF(){
 		  break;
 	  case 1:
 		  f.reset();
-		  lines=QString(f.readAll()).split(QRegExp("[\\r\\n]"),skipEmptyParts);
+		  lines=QString(f.readAll()).split(QRegularExpression("[\\r\\n]"),skipEmptyParts);
 		  for (int i=0; i<lines.size(); i++){
 		    tok=lines.at(i).split(' ',skipEmptyParts);
 		    if (!tok.isEmpty()){
@@ -227,7 +234,7 @@ void FcVsFo::readFCF(){
 		  break;
 	  case 3:{
 		   f.reset();
-		   lines=QString(f.readAll()).split(QRegExp("[\\r\\n]"),skipEmptyParts);
+		   lines=QString(f.readAll()).split(QRegularExpression("[\\r\\n]"),skipEmptyParts);
 
 		   int i= 0;
 		   for (int j=0; !j; i++){
@@ -263,7 +270,7 @@ void FcVsFo::readFCF(){
 		 break;
 	  case 4:{
 		   f.reset();
-		   lines=QString(f.readAll()).split(QRegExp("[\\r\\n]"),skipEmptyParts);
+		   lines=QString(f.readAll()).split(QRegularExpression("[\\r\\n]"),skipEmptyParts);
 
 		   int i= 0;
 		   for (int j=0; !j; i++){
@@ -297,7 +304,7 @@ void FcVsFo::readFCF(){
 		 //		 qDebug()<<"List4 OK";
 		 break;
 	  case 5:f.reset();
-		 lines=QString(f.readAll()).split(QRegExp("[\\r\\n]"),skipEmptyParts);
+		 lines=QString(f.readAll()).split(QRegularExpression("[\\r\\n]"),skipEmptyParts);
 		 for (int i=0; i<lines.size(); i++){
 		   tok=lines.at(i).split(' ',skipEmptyParts);
 		   if (!tok.isEmpty()&&(tok.size()>5)){
@@ -323,7 +330,7 @@ void FcVsFo::readFCF(){
 		 break;
 	  case 6:{
 		   f.reset();
-		   lines=QString(f.readAll()).split(QRegExp("[\\r\\n]"),skipEmptyParts);
+		   lines=QString(f.readAll()).split(QRegularExpression("[\\r\\n]"),skipEmptyParts);
 
 		   int i= 0;
 		   for (int j=0; (!j)&&(i<lines.size()); i++){
@@ -365,7 +372,7 @@ void FcVsFo::readFCF(){
 	  case 8:{
 
 		   f.reset();
-		   lines=QString(f.readAll()).split(QRegExp("[\\r\\n]"),skipEmptyParts);
+		   lines=QString(f.readAll()).split(QRegularExpression("[\\r\\n]"),skipEmptyParts);
 
 		   int i= 0;
 		   for (int j=0; (!j)&&(i<lines.size()); i++){
