@@ -32,6 +32,11 @@
 #include <QCheckBox>
 #include <QMutex>
 #include <QMutexLocker>
+#include <QToolBar>
+#include <QDialogButtonBox>
+#include <QPushButton>
+#include <QGridLayout>
+#include <QVBoxLayout>
 
 //! Rec is a reflection type of a fcf 6 file.
 typedef struct {
@@ -223,11 +228,12 @@ signals:
 #endif
     FNode *nodex,*nodey,*nodez;
     int *noGoMap;
+    QSettings *einstellung;
     Rec lr[LM];
     char cen,git;
     int nr,nc,ns;
     int mtyp,tri;
-    QTime foti,suti;
+    QElapsedTimer foti,suti;
     //          double minnor,maxnor;
     void gen_surface(bool neu,int imin=0, int imax=4);
     void CalcVertex( int ix, int iy, int iz);
