@@ -47,6 +47,7 @@
  Highlighter::Highlighter(QString FileName, QTextDocument *parent)
      : QSyntaxHighlighter(parent){
    //    setEditStyle();
+     printf("Highlighter\n");
        loadHighlightPatterns(FileName);
  }
 
@@ -59,7 +60,7 @@ void Highlighter::highlightSearch(const QString &find){
   f.replace(".*",".+");
   if ((f=="^")||(f=="$"))return;
   if (!f.isEmpty()){
-  rule.pattern = QRegExp(f,Qt::CaseInsensitive);
+  rule.pattern = QRegularExpression(f, QRegularExpression::CaseInsensitiveOption);
 
   findRule.append( rule);
   }
@@ -80,7 +81,7 @@ void Highlighter::highlightError(const QStringList fnd){
   for (int i=0;i<fnd.size();i++){
       f=fnd.at(i);
       if (!f.isEmpty()){
-	rule.pattern = QRegExp(f,Qt::CaseInsensitive,QRegExp::FixedString);
+    rule.pattern = QRegularExpression(f, QRegularExpression::CaseInsensitiveOption);//fixedString was in QRegExp
 
 	findRule.append( rule);
       }
@@ -101,7 +102,7 @@ void Highlighter::highlightDupl(const CEnvironment &dupls){
   for (int i=0;i<dupls.size();i++){
       f=dupls.at(i).orginalLine;
   if (!f.isEmpty()){
-  rule.pattern = QRegExp(f,Qt::CaseInsensitive);
+  rule.pattern = QRegularExpression(f, QRegularExpression::CaseInsensitiveOption);
 
   findRule.append( rule);
   }
@@ -113,68 +114,59 @@ void Highlighter::highlightDupl(const CEnvironment &dupls){
 void Highlighter::highlightBlock(const QString &text) {
   foreach (HighlightingRule rule, highlightingRules) {
     if (rule.Comment) continue;
-    QRegExp expression(rule.pattern);
-    int index = text.indexOf(expression);
-    while (index >= 0) {
-      int length = expression.matchedLength();
-      setFormat(index, length, rule.format);
-      index = text.indexOf(expression, index + length);
+    QRegularExpressionMatchIterator matchIterator = rule.pattern.globalMatch(text);
+    while (matchIterator.hasNext()) {
+        QRegularExpressionMatch match = matchIterator.next();
+        setFormat(match.capturedStart(), match.capturedLength(), rule.format);
     }
   }
   foreach (HighlightingRule rule, highlightingRules) {
     if (!rule.Comment) continue;
-    QRegExp expression(rule.pattern);
-    int index = text.indexOf(expression);
-    while (index >= 0) {
-      int length = expression.matchedLength();
-      setFormat(index, length, rule.format);
-      index = text.indexOf(expression, index + length);
-     // printf("in editor height = %d\n",QFontMetrics(rule.format.font()).height());
+    QRegularExpressionMatchIterator matchIterator = rule.pattern.globalMatch(text);
+    while (matchIterator.hasNext()) {
+        QRegularExpressionMatch match = matchIterator.next();
+        setFormat(match.capturedStart(), match.capturedLength(), rule.format);
     }
   }
   setCurrentBlockState(0);
-
   if (!outputstyle) {
   int startIndex = 0;
-  if (previousBlockState() != 1)
+  if (previousBlockState() != 1){
     startIndex = text.indexOf(commentStartExpression);
-
+  }
   QTextCharFormat fmt = format(startIndex);
   if (fmt.foreground()==Qt::blue) startIndex=-1;
   while (startIndex >= 0) {
-    int endIndex = text.indexOf(commentEndExpression, startIndex);
-    int commentLength;
+    QRegularExpressionMatch match = commentEndExpression.match(text, startIndex);
+    int endIndex = match.capturedStart();
+    int commentLength = 0;
     if (endIndex == -1) {
-      setCurrentBlockState(1);
-      commentLength = text.length() - startIndex;
+        setCurrentBlockState(1);
+        commentLength = text.length() - startIndex;
     } else {
-      commentLength = endIndex - startIndex
-                             + commentEndExpression.matchedLength();
+        commentLength = endIndex - startIndex
+                        + match.capturedLength();
     }
     setFormat(startIndex, commentLength, multiLineCommentFormat);
-    startIndex = text.indexOf(commentStartExpression,
-			      startIndex + commentLength);
+    startIndex = text.indexOf(commentStartExpression, startIndex + commentLength);
     fmt = format(startIndex);
     if (fmt.foreground()==Qt::blue) startIndex=-1;
     if (fmt.underlineStyle()) startIndex=-1;
   }
   foreach (HighlightingRule rule, highlightingRules) {
     if (!rule.Comment) continue;
-    QRegExp expression(rule.pattern);
-    int index = text.indexOf(expression);
-    while (index >= 0) {
-      int length = expression.matchedLength();
-      setFormat(index, length, rule.format);
-      index = text.indexOf(expression, index + length);
-     // printf("in editor height = %d\n",QFontMetrics(rule.format.font()).height());
+    QRegularExpressionMatchIterator matchIterator = rule.pattern.globalMatch(text);
+    while (matchIterator.hasNext()) {
+        QRegularExpressionMatch match = matchIterator.next();
+        setFormat(match.capturedStart(), match.capturedLength(), rule.format);
     }
   }
   }
 /*
-  rule.pattern =  QRegExp("\\b[0-9]{2,}\\.[0-9]+\\b");
+  rule.pattern =  QRegularExpression("\\b[0-9]{2,}\\.[0-9]+\\b");
   rule.format = numberFormat;
 
-  expression = QRegExp(rule.pattern);
+  expression = QRegularExpression(rule.pattern);
   index = text.indexOf(expression);
   while (index >= 0) {
     int length = expression.matchedLength();
@@ -182,12 +174,10 @@ void Highlighter::highlightBlock(const QString &text) {
     index = text.indexOf(expression, index + length);
   } */
   foreach (HighlightingRule rule, findRule) {
-    QRegExp expression(rule.pattern);
-    int index = text.indexOf(expression);
-    while (index >= 0) {
-      int length = expression.matchedLength();
-      setFormat(index, length, rule.format);
-      index = text.indexOf(expression, index + length);
+    QRegularExpressionMatchIterator matchIterator = rule.pattern.globalMatch(text);
+    while (matchIterator.hasNext()) {
+        QRegularExpressionMatch match = matchIterator.next();
+        setFormat(match.capturedStart(), match.capturedLength(), rule.format);
     }
   }
 //  printf("hi blo\n");
@@ -197,7 +187,7 @@ void Highlighter::loadHighlightPatterns(QString FileName){
   //int group=1;  
   highlightingRules.clear();
   outputstyle=true;
-  //  fprintf(stderr,"loadHighlightPatterns %s\n",FileName.toStdString().c_str());
+  //fprintf(stderr,"loadHighlightPatterns %s\n",FileName.toStdString().c_str());
   QSettings *pat = new QSettings(FileName,QSettings::IniFormat);
   /*foreach(HighlightingRule rule, highlightingRules){  
     pat->beginGroup(QString("%1").arg(group));
@@ -215,8 +205,9 @@ void Highlighter::loadHighlightPatterns(QString FileName){
     }*/
   HighlightingRule rule;
   QStringList groups = pat->childGroups();
+  QRegularExpression Dplus = QRegularExpression("\\D+");
   foreach (QString grp,groups){
-    if (!grp.contains(QRegExp("\\D+"))){
+    if (!grp.contains(Dplus)){
       pat->beginGroup(grp);
       rule.pattern.setPattern(pat->value("Pattern","").toString());
       rule.format.setForeground(QColor(pat->value("Color"           ,"#000000").toString()));
@@ -227,7 +218,7 @@ void Highlighter::loadHighlightPatterns(QString FileName){
       rule.format.setUnderlineColor(QColor(pat->value("UnderlineColor"  ,rule.format.foreground().color().name()).toString()));
       rule.format.setFontOverline(pat->value("Overlined"       ,"false").toBool());
       rule.Comment=pat->value("Comment","false").toBool();
-      rule.pattern.setCaseSensitivity((Qt::CaseSensitivity)pat->value("CaseSensitive"   ,0).toInt());
+      //rule.pattern.setPatternOptions(static_cast<QRegularExpression::PatternOptions>(pat->value("CaseSensitive"   ,0).toInt()));
       pat->endGroup();
       highlightingRules.append(rule);
     }else{
