@@ -40,7 +40,7 @@
 #define HIGHLIGHTER_H
 
 #include <QSyntaxHighlighter>
-
+#include <QRegularExpression>
 #include <QHash>
 #include <QTextCharFormat>
 #include "molecule.h"
@@ -68,7 +68,7 @@ public slots:
   bool outputstyle;  
   struct HighlightingRule
   {
-    QRegExp pattern;
+    QRegularExpression pattern;
     QTextCharFormat format;
     bool Comment;
     inline bool operator == (const HighlightingRule &a){
@@ -76,8 +76,8 @@ public slots:
   }
   };
   QList<HighlightingRule> highlightingRules,findRule;
-  QRegExp commentStartExpression;
-  QRegExp commentEndExpression;
+  QRegularExpression commentStartExpression;
+  QRegularExpression commentEndExpression;
 
 //  QTextCharFormat keywordFormat;
 //  QTextCharFormat classFormat;
