@@ -126,7 +126,7 @@ void HistoryWidget::mousePressEvent ( QMouseEvent * event ){
   //copy savefilelist.at(index) to res and open this res file
   QString dn=dirname.section('/',0,-2),
 	  argument=dirname.section('/',-1);
-  argument.remove(QRegExp(".res$|.ins$",Qt::CaseInsensitive));
+  argument.remove(QRegularExpression(".res$|.ins$",QRegularExpression::CaseInsensitiveOption));
   if (event->buttons() & Qt::RightButton){
       QMenu *menu = new QMenu("");
       QString shortname=saveFilesList.at(index).section('/',-1);
@@ -137,18 +137,18 @@ void HistoryWidget::mousePressEvent ( QMouseEvent * event ){
       a=menu->addAction(QString("Purge history. Keep newest ten entries."),this,SLOT(prune()));
       a=menu->addAction(QString("Delete %1 !").arg(shortname),this,SLOT(deleteOne()));
       a->setData(saveFilesList.at(index));
-      menu->exec(event->globalPos());
+      menu->exec(event->pos());
       delete menu;
   }
   if (event->buttons() & Qt::LeftButton){
     if (event->pos().y()<(fh+3)) {emit saveHist(); return;}
-    if (dirname.contains(QRegExp(".res$",Qt::CaseInsensitive))){
+    if (dirname.contains(QRegularExpression(".res$",QRegularExpression::CaseInsensitiveOption))){
       QString insname =dirname;
-      insname.replace(QRegExp(".res$",Qt::CaseInsensitive),".ins");
+      insname.replace(QRegularExpression(".res$", QRegularExpression::CaseInsensitiveOption),".ins");
       QString fcfname = dirname;
-      fcfname.replace(QRegExp(".res$",Qt::CaseInsensitive),".fcf");
+      fcfname.replace(QRegularExpression(".res$", QRegularExpression::CaseInsensitiveOption),".fcf");
       QString fcfname2 = dirname;
-      fcfname2.replace(QRegExp(".res$",Qt::CaseInsensitive),".2fcf");
+      fcfname2.replace(QRegularExpression(".res$", QRegularExpression::CaseInsensitiveOption),".2fcf");
       QDir work=QDir(QString("%1/%2saves/").arg(dn).arg(PROGRAM_NAME));
       QStringList filter;
       filter << QString("%1_*_.ins").arg(argument);
@@ -235,9 +235,9 @@ void HistoryWidget::setPath(QString dirName,int wid){
   dirname=dirName;
   QString dn = dirName.section('/',0,-2),
 	  argument=dirName.section('/',-1);
-  argument.remove(QRegExp(".res$|.ins$",Qt::CaseInsensitive));
+  argument.remove(QRegularExpression(".res$|.ins$",QRegularExpression::CaseInsensitiveOption));
   QString insname =dirName;
-  insname.replace(QRegExp(".res$",Qt::CaseInsensitive),".ins");
+  insname.replace(QRegularExpression(".res$",QRegularExpression::CaseInsensitiveOption),".ins");
 
   QDir work=QDir(QString("%1/%2saves/").arg(dn).arg(PROGRAM_NAME));
   QStringList filter;
@@ -389,8 +389,13 @@ void HistoryWidget::paintEvent(QPaintEvent *event){
       x = qMax(FreeWerte.value(i),x);
   }
   QPainter p(this);
+#if (QT_VERSION >=0x050000)
+  setMinimumWidth(qMax(p.fontMetrics().horizontalAdvance(QString("%1").arg(x,4,'f',3))*RWerte.size(),maxwid));
+  setMaximumWidth(qMax(p.fontMetrics().horizontalAdvance(QString("%1").arg(x,4,'f',3))*RWerte.size(),maxwid));
+#else
   setMinimumWidth(qMax(p.fontMetrics().width(QString("%1").arg(x,4,'f',3))*RWerte.size(),maxwid));
   setMaximumWidth(qMax(p.fontMetrics().width(QString("%1").arg(x,4,'f',3))*RWerte.size(),maxwid));
+#endif
   p.fillRect(0,0,width(),height(),QPalette().window());// );QColor("#eeeecc")
   p.setPen(Qt::black);
 
