@@ -8,15 +8,12 @@
 class IdealDefdenGenerator : public QThread{
 Q_OBJECT
   public:
-  IdealDefdenGenerator(ChGL *chgl_,FourXle *FX, QObject * parent = 0 ){
+  IdealDefdenGenerator(ChGL *chgl_,FourXle *FX, QObject * parent = nullptr ){
     chgl = chgl_;
     mommy = parent;// maybe I need it later?
     fxle=FX;
-  }
-  ~IdealDefdenGenerator(){
-     // printf("destruct ideal\n");
-  }
-  void run() {
+  }  
+  void run() override {
     /* expensive or blocking operation  */
     beloWriteCube();
   }
