@@ -505,15 +505,15 @@ QString XLOutputParser::residualsTextDisplay(QString &line, int ls_cycles) {
                          "<tr><td align=right>Highest peak = </td> <td> %3 e&Aring;<sup>-3</sup> at %4 &Aring; from %7 &nbsp;&nbsp;&nbsp;&nbsp;</td>"
                          "    <td align=right>Deepest hole = </td> <td> %5 e&Aring;<sup>-3</sup> at %6 &Aring; from %8 </td></tr>"
                          )
-          .arg(listf_data.R1_4sig.toDouble()*100, 4, 'f', 2, 0)      // 1
-          .arg(listf_data.finalwR2.toDouble()*100, 4, 'f', 2, 0)     // 2
+          .arg(listf_data.R1_4sig.toDouble()*100, 4, 'f', 2, '0')      // 1
+          .arg(listf_data.finalwR2.toDouble()*100, 4, 'f', 2, '0')     // 2
           .arg(listf_data.peak)           // 3
           .arg(listf_data.peakdist)       // 4
           .arg(listf_data.hole)           // 5
           .arg(listf_data.holedist)       // 6
           .arg(listf_data.peakfrom)       // 7
           .arg(listf_data.holefrom)       // 8
-          .arg(listf_data.R1_all.toDouble()*100, 4, 'f', 2, 0)       // 9
+          .arg(listf_data.R1_all.toDouble()*100, 4, 'f', 2, '0')       // 9
         ;
     // One of flack or parson exists:
     if (!listf_data.flack.isEmpty() || !listf_data.parson.isEmpty()) {
@@ -655,7 +655,7 @@ QString XLOutputParser::residualsTextDisplay(QString &line, int ls_cycles) {
 QString XLOutputParser::xl_version(QString line) {
   //! Returns the running shelxl version
   //! " +  Copyright(C) George M. Sheldrick 1993-2014     Version 2014/7  +"
-  QRegExp versionregex = QRegExp("Version\\s+\\d+\\/\\d+");
+  QRegularExpression versionregex = QRegularExpression("Version\\s+\\d+\\/\\d+");
   QString version;
   QStringList linesplit;
   if (line.contains(versionregex)) {
@@ -673,7 +673,7 @@ QStringList XLOutputParser::xl_threads(QString line) {
   QString threads = "";
   QString processors = "";
   QStringList linesplit;
-  QRegExp tprocregex = QRegExp("Running\\s+\\d\\s+threads\\s+on\\s+\\d\\s+processors");
+  QRegularExpression tprocregex = QRegularExpression("Running\\s+\\d\\s+threads\\s+on\\s+\\d\\s+processors");
   if (line.contains(tprocregex)) {
     linesplit = line.split(" ", skipEmptyParts);
     threads = linesplit.at(1);
@@ -735,11 +735,11 @@ QStringList XLOutputParser::xl_rint_rsigma(QString line) {
   QString l=line.remove('=');
   if (line.contains("R(sigma)")) {
     linesplit = l.split(" ", skipEmptyParts);
-    int RintInd = linesplit.indexOf(QRegExp("R\\(int\\)"));
+    int RintInd = linesplit.indexOf(QRegularExpression("R\\(int\\)"));
     if ((RintInd != -1) && linesplit.length() > RintInd+1) {//>= leads to segmentation fault
       rint = linesplit.at(RintInd+1);
     }
-    int RsigInd = linesplit.indexOf(QRegExp("R\\(sigma\\)"));
+    int RsigInd = linesplit.indexOf(QRegularExpression("R\\(sigma\\)"));
     if ((RsigInd != -1) && linesplit.length() > RsigInd+1) {//>= leads to segmentation fault
       rsigma = linesplit.at(RsigInd+1);
     }//else qDebug()<<linesplit<<RsigInd;
@@ -843,7 +843,7 @@ QStringList XLOutputParser::xl_current_max_shift(QString line) {
 
 QString XLOutputParser::xl_current_warning(QString line) {
   //! Returns a Warning if there is any in the current line.
-  QRegExp warnregexp = QRegExp("\\*\\*");  //.*\\*\\*");
+  QRegularExpression warnregexp = QRegularExpression("\\*\\*");  //.*\\*\\*");
   QString warn = "";
   if ( line.contains(warnregexp) ) {
     warn = line.simplified();
@@ -853,7 +853,7 @@ QString XLOutputParser::xl_current_warning(QString line) {
 
 void XLOutputParser::xl_collect_warnings(QString line) {
   //! Returns all warning that occoured during refinement.
-  QRegExp warnregexp = QRegExp("\\*\\*");  //.*\\*\\*");
+  QRegularExpression warnregexp = QRegularExpression("\\*\\*");  //.*\\*\\*");
   if ( line.contains(warnregexp) ) {
     warnings.append(line.simplified());
   }
