@@ -19,13 +19,13 @@
 #ifndef MOLECULE_H
 #define MOLECULE_H 1
 #include <math.h>
+#if defined _MSC_VER &&  _MSC_VER == 1500
 #ifndef fmin
 #define fmin(x, y) (((x) < (y)) ? (x) : (y))
 #endif
 #ifndef fmax
 #define fmax(x, y) (((x) > (y)) ? (x) : (y))
 #endif
-#if defined _MSC_VER &&  _MSC_VER == 1500
 #ifndef round 
 #define round(x) (x<0?ceil((x)-0.5):floor((x)+0.5))
 #endif
@@ -34,14 +34,14 @@
 #include <QStringList>
 #include <QList>
 #include <QtOpenGL>
-#include <QGLShaderProgram>
+#include <QOpenGLShaderProgram>
 //#if defined (Q_WS_MAC) || defined(Q_OS_MAC)
 //#include <OpenGL/glu.h>
 // //#include <glu.h>
 //#else
 //#include <GL/glu.h>
 //#endif
-#include "glureplace.h"
+//#include "glureplace.h"
 #include <QtCore>
 #ifndef M_PI
 #define	M_PI		3.14159265358979323846
@@ -265,6 +265,19 @@ inline Matrix operator + (const Matrix &a, const Matrix &b){
   erg.m33 = a.m33+b.m33;
   return erg;
 }
+
+inline bool operator < (const Matrix &a, const Matrix &b){
+    return  (
+            (a.m11<b.m11)&&
+            (a.m21<b.m21)&&
+            (a.m31<b.m31)&&
+            (a.m12<b.m12)&&
+            (a.m22<b.m22)&&
+            (a.m32<b.m32)&&
+            (a.m13<b.m13)&&
+            (a.m23<b.m23)&&
+            (a.m33<b.m33));
+}
 //! MyAtom an atom object
 typedef struct MyAtom{
   //! Label is the atom label with residue nuber seperated by '_' and the symmetry operation number seperated by french quotes.
@@ -360,8 +373,14 @@ struct Vert {
 typedef struct VPoly{
  V3 mid,nor,verts0,verts1;
  int acol,intra;
- V3 glide;
 }VPoly;
+
+typedef struct PPoly{
+ V3 mid,nor,verts0,verts1;
+ int colr,n;
+ V3 glide;
+}PPoly;
+
 typedef QList<MyBond> Connection;
 typedef QList<MyAtom> CEnvironment;
 //! Unit cell parameters
@@ -463,9 +482,12 @@ class Ring;
  * It is thought that there is only one instance of this class in ShelXle during runtime. Molecule accesses the settings file of ShelXle.
  */
 class Molecule{
-  public:
+public:
     GLfloat fgc[4];
     bool mist;
+    QString theIconPath;
+    float fibo,fibom1;//1.618034f
+    GLfloat q_bondWidth = 1.0f;
     int fogrange;
     bool nopm1;//!< No part minus n ghost
     bool highlightEquivalents;//!< Symmetry generated objects will be drawn in lighter colors if this is true
@@ -483,7 +505,7 @@ class Molecule{
     QString titl;
     QString mynewnameis;
     int LOD;//!< The Level of Detail
-    QGLShaderProgram *g_Program,*saveProg;//shaders with Qt
+    QOpenGLShaderProgram *g_Program,*saveProg;//shaders with Qt
     int g_program;//GL handle for shader program
     Molecule();//!<The constructor
     int icocnt;
@@ -526,15 +548,18 @@ class Molecule{
     bool   beloColorByDefden; 
     bool   beloLabels;
     Matrix rotarb(double *arr);
-    GLuint hbtex; //!< texture handle for hydrogen bonds
+    //GLuint hbtex; //!< texture handle for hydrogen bonds
+    QOpenGLTexture *hbtex = nullptr;
     //	 hbtex2; //!< texture handle for
-    GLuint adpwall,//!< texture handle elipsoid walls
-           adpwall_plaid;//!< texture handle elipsoid walls different style
+    //GLuint adpwall,//!< texture handle elipsoid walls
+    //       adpwall_plaid;//!< texture handle elipsoid walls different style
     // int nonPositiveDefinite;
     int proba,//!< probability level of the ellipsoids
         tubes,//!< tube mode instead of ball stick 
         dratom;//!< wire style of atoms
-    //V3 VZ;
+    //V3 VZ;    
+    QColor deuteriumColor;
+    QColor atomColor(int an, bool isD=false);
     QColor AtomColor[109];//!<List of colors for all supported elements.
     int AtomStyle[109];//!<List of atom style types for all supported elements.
     double arad[109];//!<List of ball radii for ball stick visulaization  for all supported elements.
@@ -564,34 +589,50 @@ class Molecule{
     Connection theH_Bonds;
     QStringList usedSymmetry;//!< List of internal symmetry codes of symmetry operations curently applied.
     QString HumanSymmetry;//!< A human readable list of symmetry operations in use.
-    QSettings *einstellung;//!< the QSetttings for ShelXle.
+    //QSettings *einstellung;//!< the QSetttings for ShelXle.
     QString ffmpegexe;
     int vorobas;
     QString voroMsg;
+    V3 findGlide(V3 normal, V3 test, int n);
+    V3 findGlide(V3 normal, V3 test, Matrix mat, V3 trans);
     QList<VPoly> vtriangles;
-    QList<V3> axvecs;
+    double planeTransparence;
+    QList<PPoly> ptriangles;
+    QList<V3>  axvecs;
+    QList<int> axcols;    
+    QList<int> axoprs;
+    QList<int> invoprs;
+    QList<V3>  invvecs;
+    QList<int> dontshowSop;
+    QList<int> dontshowSel;
+    QList<QColor> symmColors;
+    void spitze(V3 dir, V3 pos);
     void voronoij(CEnvironment au, int intat=-1);
-    void drawVoronoi(V3 auge);
-    void drawAchses();
+    void drawVoronoi(V3 auge);    
+    void drawPlanes();
+    void drawAxses();
+    void drawInversionCenters();
     double ueq(Matrix m);
     double averageUeq();
     V3 intersectPoint(V3 rayVector, V3 rayPoint, V3 planeNormal, V3 planePoint, bool &ok);
-    void make1Plane(V3 planeNormal, V3 pointOnPlane,V3 glide, int ac);
+    void make1Plane(V3 planeNormal, V3 pointOnPlane,V3 glide, int ac, int n);
     void findSymmElements();
-    void packPoint(const V3 &point, V3 &normal, QList<V3> &wearein);//!< Pack inside cell
-    void packLine(const V3 &p1,const V3 &p2, V3 &vec, QList<V3> &wearein);//!< Pack inside cell
+    void packPoint(const V3 &point, QList<V3> &wearein);//!< Pack inside cell
+    void packLine(const V3 &p1,const V3 &p2, QList<V3> &wearein, int c,QList<int> &dc);//!< Pack inside cell
+    void specialPts(const Matrix &mat, const V3 &trans, QList<V3> &points);
     //unsigned short ElNeg[83];
     //
     //
+    QString nameSymmOp(Matrix m, int &fold, V3 &axis);
     QStringList sdmcompleter();
     QList<SdmItem> computeSDM(CEnvironment au);
     void atoms(CEnvironment atom,int proba=50,double radscal=1.0);
     bool shaders_work;//!< can glsl shader run on this machine?
-    bool program_in_use;//!< glsl program active 
-    bool useShaders;//!< the user whats shader
+    bool program_in_use;//!< glsl program active
+    //bool useShaders;//!< the user whats shader
     void shaderAtoms(CEnvironment atom,int proba=50, double radScal=1.0);//!< like void atoms(CEnvironment atom,int proba=50); but with shaders
     void pappe();//!< the inner walls of an ellipsoid (use ony with shaders and 'wall' uniform set)
-    void bonds(Connection bond);
+    void bonds(Connection bond, double radScal=1.0);
     void dbond(Connection bond,int ccode=0);
     void lbond();
     void billBoard();
@@ -647,23 +688,22 @@ class Molecule{
     const QStringList thepse(){return PSE;}
     int pseSize(){return PSE.size();}
     Matrix u2b(Matrix u);//!< compute Bij values from Uij 
-    Matrix b2u(Matrix u);//!< compute Uij values from Bij 
-    void extendChain(QList<Ring > &rings, QList<int> &currentChain, QList<QList<int> > &neighbors,QSet<int> &nogo);
-
-    void inventNewLabels(QList<int> &result);//!< lets see where it goes
+    Matrix b2u(Matrix u);//!< compute Uij values from Bij
     double * jacobi(const Matrix &uij, V3 &ev);
     void exportOBJ(QString fileName,const CEnvironment &atoms, const Connection &_bonds);
 
     int rankMySymmOp(Matrix m, V3 t);
+    void message(const QString &);//!< small text messages to the status bar.
   private:
     void neighborSort(Knopf &kn);
     bool checkExtensions();
     double HAMax,HAWink;
-    void ellipse(int style);
+    //void ellipse(int style);
     void hirsh(Connection bond);
     void sphere(int adp,int style=7);
     void mySphere(int bal);
     void myCylinder(int rod);
+    void myCone();
     void cylinder(double strength, double length);
     //  void ikosa(double R);
     //  V3 eigenvalues(const Matrix m);
