@@ -130,7 +130,11 @@ void RenderPlot::drawScale() {
   foreach (mypoint p, datapoints) {
     ylist.append(p.y);
   }
+#if (QT_VERSION >= 0x050000)
+  std::sort(ylist.begin(), ylist.end());
+#else
   qSort(ylist.begin(), ylist.end());
+#endif
   distx = width() / (mcycles+1);
   // 14 is an empirical number and defines extra space on the y scale:
   if (ylist.length() > 1) {
