@@ -2,6 +2,9 @@
 #define SAVEHISTORYWIDGET_H
 
 #include <QtGui>
+#include <QDialog>
+#include <QPushButton>
+#include <QLabel>
 #include "molecule.h"
 #include "itsme.h"
 #include "codeeditor.h"
