@@ -3,22 +3,22 @@
 #define SHX_HELPER_H
 #include <QtCore>
 QStringList shxToolTips =(QStringList()<< 
-"TITL [ ]"<<
+"TITL [ ]  Title of up to 76 characters"<<
 "CELL wavelength[A] a[A] b[A] c[A] alpha[°] beta[°] gamma[°]"<<
 "ZERR Z esd(a) esd(b) esd(c) esd(alpha) esd(beta) esd(gamma)"<<
-"LATT N  N= 1=>P, 2=>I, 3=>R, 4=>F, 5=>A, 6=>B, 7=>C. "<<
+"LATT N  N= 1=>P, 2=>I, 3=>R, 4=>F, 5=>A, 6=>B, 7=>C. Negative N=> non-centrosymmetric"<<
 "SYMM symmetry operation"<<
 "SFAC elements or \nSFAC label a1 b1 a2 b2 a3 b3 a4 b4 c f' f'' mu r wt"<<
 "DISP E f' f'' [#] mu [#]"<<
-"UNIT n1 n2 ..."<<
+"UNIT n1 n2 ... \nNumber of atoms of each type in the unit-cell"<<
 "LAUE E (values of f' and f\")"<<
 "REM the rest of the line is a comment"<<
-"MORE m ,sets the verbosity m=0..3"<<
+"MORE m[1] ;Sets the verbosity m=0..3\nNegative m writes a covariance .mat file."<<
 "TIME t[s]\nHas been deprecated!"<<
 "HOPE\nHas been deprecated!"<<
 "END end of input"<<
 "HKLF N[0] S[1] r11...r33[1 0 0  0 1 0  0 0 1] sm[1] m[0]"<<
-"OMIT s[-2] 2Theta(lim)[180] or\nOMIT h k l"<<
+"OMIT s[-2] 2Theta(lim)[180] or\nOMIT h k l\nOMIT atomnames e.g. $H"<<
 "SHEL lowres[infinite] highres[0]"<<
 "BASF scale factors"<<
 "TWIN 3x3 matrix [-1 0 0 0 -1 0 0 0 -1] n[2]"<<
@@ -39,11 +39,11 @@ QStringList shxToolTips =(QStringList()<<
 "CONN bmax[12] r[#] atomnames or \nCONN bmax[12]"<<
 "BIND atom1 atom2"<<
 "FREE atom1 atom2"<<
-"DFIX d s[0.02] atom pairs"<<
-"DANG d s[0.04] atom pairs"<<
+"DFIX d s[0.02] pairs of atoms"<<
+"DANG d s[0.04] pairs of atoms"<<
 "BUMP s [0.02]"<<
-"SAME s1[0.02] s2[0.02] atomnames"<<
-"SADI s[0.02] atom pairs"<<
+"SAME s1[0.02] s2[0.04] atomnames"<<
+"SADI s[0.02] pairs of atoms"<<
 "CHIV V[0] s[0.1] atomnames"<<
 "FLAT s[0.1] four or more atoms"<<
 "DELU s1[0.01] s2[0.01] atomnames"<<
@@ -60,7 +60,7 @@ QStringList shxToolTips =(QStringList()<<
 "WGHT a[0.1] b[0] c[0] d[0] e[0] f[.33333]"<<
 "FVAR osf[1] free variables"<<
 "BOND atomnames"<<
-"CONF atomnames"<<
+"CONF atomnames max_d[1.9] max_a[170]"<<
 "MPLA na atomnames"<<
 "RTAB codename atomnames"<<
 "HTAB dh[2.0]\nHTAB donor-atom acceptor-atom"<<
@@ -87,13 +87,13 @@ QStringList shxToolTips =(QStringList()<<
 "ADDA O2T_999 etc. add atoms to residue so that H-restrains are obeyed."<<
 "STAG torsion angle restraint for staggered"<<
 "NEUT convert x-ray SFAC table to neutrons"<<
-"ABIN n1 n2  including precalculate structure factors"<<
+"ABIN n1 n2; Reads name.fab with precalculated structure factors."<<
 "ANSC six coefficients\nanisotropic scaling"<<
 "ANSR [0.001] restraint for ANSC "<<
 "NOTR p[170] d[1.9] removes torsion angles generated by CONF from the .cif file"<<
 "TWST N[0] specify twin component used for completeness and Friedel statistics"<<
 "BEDE atom1 atom2 r A U1 U2"<<
-"LONE m atomnames A U1 U2 r phi\nphi required for m = 2, 3, 7, 9, and 12"//<<
+"LONE atomnames m A U1 U2 r phi\nphi required for m = 2, 3, 7, 9, and 12"//<<
 );
 
 
