@@ -36,10 +36,10 @@
 #ifndef nullptr 
 #define nullptr NULL
 #endif
+#if defined _MSC_VER &&  _MSC_VER == 1500
 #ifndef fmin
 #define fmin(x, y) (((x) < (y)) ? (x) : (y))
 #endif
-#if defined _MSC_VER &&  _MSC_VER == 1500
 #define round(x) (x<0?ceil((x)-0.5):floor((x)+0.5))
 #endif
 #undef vc7hack1
@@ -73,14 +73,21 @@
 class Window : public QMainWindow {
   Q_OBJECT
   public:
-    Window();
+    Window(ChGL *_chgl);
     virtual ~Window();
     //bool scroller;
     static int Version();//!< This return the actual Revision
+    QString theIconPath;
+    QString CurrentCellString;
     bool restraintsOrConstraints();
     int MaxNumberOfPeaks;
+    int wght_max_cycls;
     static QString programName(){return QString(PROGRAM_NAME);}//!<get the actual programname
-    ChGL *chgl;  
+    ChGL *chgl;
+    QSettings *einstellung;
+#if (QT_VERSION >=0x050000)
+    QScreen *screen;
+#endif
     QDockWidget *dock, *renamDock, *qPeakDock, *histDock;
     QDockWidget *shxEditorDock,*shxOutputDock,*infotainmentDock;
     QAction *enterRenameMode;
@@ -117,17 +124,23 @@ class Window : public QMainWindow {
     QAction *togHBond;
     QAction *togBackGr;
     QAction *togDepthCueing;
+    QAction *actionPovRay;
+    QAction *actionExportCC;
     QAction *killQs;
     QAction *killHs;
     QAction *astdlg;
     QAction *equivHighlighting;
     QAction *wghtAct;
     QAction *sinaAct;
+    QAction *squeezeAct;
     QAction *recalcSDM;
     QAction *renamUpdateAct;
     QAction *cntrPlot;
     QAction *useNativeDlg;
     QAction *darkTheme;
+    QAction *switch2HKLF5;
+    QAction *switch2HKLF4;
+    QAction *symmElementsAct;
     QTextBrowser *brs;
     QProcess *runffmpeg;
     QLineEdit *ffmpegedt,*mp4file;
@@ -151,19 +164,26 @@ class Window : public QMainWindow {
           *viewMenu, //!< View menu 
           *packMenu, //!< Packing, growing, fusing etc...
           *extraMenu;//!< add and start external programs...
+
+    QCheckBox *qGrowPeak;
+    QCheckBox *search4duplicates;
     QToolButton  *zoomIn;
     QToolButton  *zoomOut;
     QToolButton  *homeMe;
     QToolButton *quickReLoadButton;
-    QCheckBox *sr_same;
-    QCheckBox *sr_simu;
-    QCheckBox *sr_rigu;
+    QCheckBox *sr_same;//use same restraint for part atoms
+    QCheckBox *sr_simu;//use sumu restraint for part atoms
+    QCheckBox *sr_rigu;//use rigu restarint for part atoms
+    QCheckBox *sr_alter_name;//give alternataive name to 2nd part atoms
+    QCheckBox *sr_ExyzAadp_ifclose; // apply EXYZ and EADP if atoms are very close
     bool notRefine;
     int tbiconSize;
     QHash<QString,QVariant> myHash;//!< This Hash connects recent file names with cursor positions. So that files are opened always at the same position.
     QAction *refineAct;
     QAction *refineActAnis;
     QAction *refineActActa;
+    QAction *refineActExti;
+    QAction *refineActConf;
     QAction *clearVoro;
     FourXle *fxle;//!<The fourier map object.
     //  QList<V3> orte;
@@ -175,10 +195,12 @@ class Window : public QMainWindow {
     QMap<int,int> fvarCntr;//!<Free Variable counter QMap.
     bool isLabelInUse();//!< Checks if the current Label in the reaname dialog is in use. \returns true if Label is already in use.
     QString dirName; //!< Full path of the currently opened file (including file name).
+    QString lastDirName;
     QString actionInfo;
     void setFocus(bool b){focusSet=b;}
     bool isFocusSet() {return focusSet;}
     int getFvarCntr(int fvar);//!< returns the number of uses of a free variable.
+    int getFvarNr(double v, const QList<double> fv);
     int firstAtomLine;
     int extraIndex;
     bool finWasPressed;
@@ -186,6 +208,7 @@ class Window : public QMainWindow {
     QAction *ydleStop;
     QSpinBox *lpart;
     QSpinBox *rpart;
+    bool canWeSqueeze();
 
 signals:
     void fileLoaded(void);
@@ -193,8 +216,8 @@ signals:
     void movedByUser();
 
     public slots:
-      void qinitres();
-    void addNewScatteringFactor(int oz);//!<Adds a new scattering factor to the SFAC / UNIT list.
+    void qinitres();
+    void addNewScatteringFactor(int oz, bool HisD);//!<Adds a new scattering factor to the SFAC / UNIT list.
     void addDissorderedMethyl();//!<Adds rotation dissordered hydroghen atoms to a R-C carbon atom. (methyl group) AFIX 127.
     void addMethyl();//!<Adds hydroghen atoms to a R-C carbon atom. (methyl group) AFIX 137.
     void addHydroxyl();//!<Adds hydroghen atoms to a O-X oxygen atom. (hydroxyl group) AFIX 148.
@@ -214,17 +237,20 @@ signals:
     void changeElemetofLabel(const QString &pse);//!<NextLabel of rename mode consists of labelPSE labelIndex labelSuffix, this changes the LabelPSE to pse. @param pse next labelPSE. 
     void changeElement();//!<Changes the element of an atom (chgl->ImeanThisAtom) to an other scattering factor (acation->data)
     void changeIconSize();//!<Changes the size of the tool bar icons.
+    void changeInvRad(double r);
+    void changePlaneTransparence(double t);
     void changeQPeakLimit();//!<Maximum number of Q-peaks created by addMoreQPeaks
     void changeIndexofLabel(const int i);//!<NextLabel of rename mode consists of labelPSE labelIndex labelSuffix, this changes the LabelIndex to pse. @param i next labelIndex. For i < 0 no number in label. 
     void changeLabelFont();//!<Changes the font of the on screen Labels of the atoms.
     void changeResiPart();//!< change the resi or part of selected atoms.
     void changeScalefactor();//!<Changes the screen shot scale factor. 
-    void changeSuffixofLabel(const QString &fix);//!<NextLabel of rename mode consists of labelPSE labelIndex labelSuffix, this changes the labelSuffix @param fix next labelSuffix.
+    void changeSuffixofLabel(int);//!<NextLabel of rename mode consists of labelPSE labelIndex labelSuffix, this changes the labelSuffix @param fix next labelSuffix.
     void checkForUpdates();//!<Checks via the internet if new versions of ShelXle are availabe.
     void clipboardDataChanged();//!<Enables the paste action if something is in the clipboard.
     void clearStructure();
     void closeEvent(QCloseEvent *event);//!<Tidys everything up and stores settings before closing the ShelXle app.
     void configBeLos();//!< Configure BEDE LONE Objects style
+    void connectEdior();
     void darkEdit();
     void brwsEPS();
     void everyPointOne();
@@ -254,6 +280,7 @@ signals:
     void fileNative();
     bool fileSave(bool spell=true, bool loadafter=true);//!<saves the editor content to the current file. @param spell disable spell checking during save @param loadafter disable reloading of file.
     bool fileSaveAs();//!<Saves the editor content to a specified file.
+    void fileSave4hklf5();
     void fillCell();//!<Uses symmetry until the unit cell box is full.
     void showSearch();//!< shows the search and repleace for the editor.
     void findNext();//!<Find next (F3) functionality of search and replace in the editor. 
@@ -264,6 +291,8 @@ signals:
     void fixIt();//!< Fix some parameters from being refined.
     void foversusfc();//!< Shows a duialog to analyse and omit outlier reflections.
     void fuse();//!< show only the asymetric unit.
+    void go2hklf5();
+    void go2hklf4();
     void grossRC(const QString &text);//!< Capitalizes the text and sets the resiResiClass text to text @param text new text for resiResiClass.
     void grow();//!< completes the molecule by symmetry.
     void growQPeaks(bool qpg);//!< applies symmetrie operations used for the main structure also for the Q-Peaks.
@@ -291,7 +320,7 @@ signals:
     void loadLst();//!< loads the .lst file if it exists.
     void loadrefres();//!< loads the refinement results from the res file.
     void loadThisFile();//!< loads a file from an QAction with path as Action data. 
-
+    void makeItHKLF5();
     void makeVoro();
     void clearVoros();
     void moveSymmMate();//!< Moves the specified symmetry generated atom to the asymmetric unit.
@@ -322,12 +351,15 @@ signals:
     void rufmode();//!< new labels can not be duplicates if this mode is on
     void runShelXL();//!< copies res to ins file and runs shelxl 
     void runExtra();//!< runs rexternal programs
+    void runSqueeze();
     void runXLWGHT();//!< runs shelxl for convergeWght. 
+
     void saveScene();//!< saves a srceen shot of the OpenGL widget
     void savePovRay();
     void savePovInc(QString povName);
     void sdm();//!< the shortest distance matrix is (re)calculated and the structure is grown
     void selectInStructure(const QString &info);//!< selects all atoms found in selected text in the structure @params info seleted text. 
+    void setCalculatorsCurrentCell();
     void shredCIF();//!< runs shredcif and loads ins/res file
     void selectParts();//!< selects atoms for 'Part Selector'
     void setShelxlOptions();//!< sets path and command line options for ShelXl executable.
@@ -335,9 +367,11 @@ signals:
     void setExtraPath();//!<sets path for an external program
     void setDefaultPath();//!<when the first recent file does not exists then open dialog starts here
     void selOmitG5();
-    void selOmitG10();
+    void selOmitG10();    
     void updateExtraIcon();//!< redraws the icon preview
+    void sColorDLG();
     void setSDMLimit();//!< max atoms in au to skip sdm calculation
+    void setMaxWGHTCycls();//!< maximum cycles in WGHT refine
     void setMonoQ(bool vis);//!<sets monochome modus for Q-Peaks to vis @param vis mochrome mode of Q-Peaks.
     void shxActUpdate(bool vis);//!< the editor tool bar gets visible when the editor is visible.
     void shxOPTUpdate(bool vis);//!< the editor tool bar gets visible when the editor is visible.
@@ -348,6 +382,7 @@ signals:
     void splitRotate();
     QString strippedName(const QString &fullFileName);//!< the file name without the path is retrned \returns the filename without path. @param total file path. 
     void styleDialog();//!< Atom style dialog is shown.
+    void symmElements(bool b);
     void symmetryManager();//!< a symmetry manager is shown
     void testMerohedral();
     void toggDocWid(bool weg);//!< hides Text Windows
@@ -355,6 +390,8 @@ signals:
     void toggleFullScreen();
     void toggleHFSAO(bool on);
     void togOMIT(bool b);
+    void togSop(bool b);
+    void togSel(bool b);
     void toggleOrtho(bool ov);//!< toggles orthographic and perspective projection.
     void undoRename();//!<if undo is clicked and it was a rename a rename is undone.
     void uniq();//!< moves centers of gravity of all molecules into the unit cell box. !does not heal scrambled molecules!
@@ -402,7 +439,10 @@ signals:
     void moveEvent(QMoveEvent *event);
     void dragEnterEvent(QDragEnterEvent *event);
     void dropEvent(QDropEvent *event);
+    void showEvent(QShowEvent * event);// override;
+    void hideEvent(QHideEvent * event);// override;
   private:
+    friend class AulaWorker;
     bool focusSet;
     bool fullscreen;
     int wght_cycls; //!< Number of cycles during the wght refinement.
@@ -420,6 +460,7 @@ signals:
     QSpinBox *symmFrgBx;
     QLineEdit *prosaSymm;
     QTimer prosaTimer;
+    QTimer *tim;
     QTextBrowser *OrtepCodesbrowser;
     QLabel *shortestDistanceLabel;
     QPushButton *willpse;
@@ -459,6 +500,7 @@ signals:
     QList<int> undoAtomIndex,redoAtomIndex,gehtMichAn,gehtMichAn2;
     int altursize;
     int mode;
+    void getMinMaxSintl(double &smin, double &smax);
     QMenu *recentFilesMenu,*stereoMenu,*moveMenu,*helpMenu,*sortMenu;
     QActionGroup *stereoGroup,*tubeGroup,*sortGroup;
     enum { MaxRecentFiles = 20 };
@@ -471,7 +513,7 @@ signals:
     QDoubleSpinBox *cStepBx;
     QDialog *scalDlg;
     QTextBrowser *infoKanal;
-    QRadioButton *pserbt[110];
+    QRadioButton *pserbt[111];
     QGroupBox *sfacBox;
     double scalePic;
     QString saveName,fvarinfo;
@@ -481,7 +523,8 @@ signals:
     QString shelxlPath, shelxlArgs;
     bool useMyBArray;    //! If set to true, the b-array is not automatically set by ShelXle
     QStringList extraProgramNames,extraProgramPaths,extraOptions,extraExt2;
-    QList<Qt::CheckState>  extraArguments,extraExtensions, extraDetach, extraResIns, extraOpenFileDialogBefore, extraIconOverlay;
+    QList<Qt::CheckState>  extraArguments,extraExtensions, extraDetach, extraResIns, extraOpenFileDialogBefore, extraIconOverlay, extraAbsolutePath;
+
     QLineEdit *extraPath,*extraName,*extraAltExtEdit,*extraArgumentsEdit,*shxPath,*shxOptions;
     QList<int> extraOverlayPointSize;
     QSpinBox *iovlPointSize;
@@ -502,14 +545,18 @@ signals:
     QLabel *nliL,*resiResiClassL,*resiNrSpinL,*partSpinL;         
     QHBoxLayout *sfacla;
     QScrollArea *qsa,*qsa2;
-    QAction *zm_stereo, *pe_stereo, *xe_stereo, *an_stereo, *hw_stereo, *no_stereo; 
+    //QAction *zm_stereo;
+    QAction *pe_stereo;
+    QAction *xe_stereo;
+    QAction *an_stereo;
+    //QAction *hw_stereo;
+    QAction *no_stereo;
     QAction *fcvsfoAct, *calcAct, *dsrAct;
     QAction *soact;
     QString shredCIFPath;
     QAction *backColor,*labelColor;
     QVBoxLayout *lt;
     QLabel *sLabel, *fvused;
-    QCheckBox *qGrowPeak,*search4duplicates;
     QTimer *hidwillpse;
 
     QLabel *indexL,*sufixL;
@@ -531,6 +578,7 @@ signals:
     QIcon extraIcon;
     QCheckBox *useArgument;
     QCheckBox *omitExtension;
+    QCheckBox *absolutePath;
     QCheckBox *detach;
     QCheckBox *copyResIns; 
     QCheckBox *openFileDialogBefore; 
@@ -554,7 +602,9 @@ signals:
     MyAtom findOH(V3 donor, V3 acceptor,int dindex,QStringList alab);
     bool maybeSave();  
     bool maybeSave2();
+    bool doWeHaveHKLF5here(QStringList &hk5s);
     bool startFused;
+    QStringList hklf5files;
     Calculator *calcmenu;
     DSRGui *dsr;
     QString pngName;//output file of povray export
@@ -572,5 +622,34 @@ signals:
     //QString selectedRestraintsAtoms(QString buffer, QList<MyAtom> selected, QString resiSpec, bool exclude_H=false);
 
 };
+class AulaWorker : public QThread {
+Q_OBJECT
+    public:
+    AulaWorker(Window *parent, bool ur){
+        _w = parent;
+        useResis = ur;
+        rzeit.start();
+    }
+    void run() override {
+      /* expensive or blocking operation  */
+      aula();
+    }
+    Window *_w;
+    bool saveruf;
+    int possible;
+    QList<int> chain;
+    void aula();
+    bool useResis;
+public slots:
+    void finalize();
+signals:
+    void message(const QString &);//!< small text messages to the status bar.
+    void rename(int);
+private:
+    QElapsedTimer rzeit;
+    QString all;
+    void extendChain(QList<Ring > &rings, QList<int> &currentChain, QList<QList<int> > &neighbors,QSet<int> &nogo);
+    void inventNewLabels(QList<int> &result);
+};
 #endif
 
